function mstat_cb(action)

app_version='1.5';
app_date='02.01.2023';

Array_id=findobj(0,'Tag','ArrayView');
ArrayAxes_id=findobj(Array_id,'Tag','ArrayAxes');
ArrayImage_id=findobj(get(ArrayAxes_id,'Children'),'Tag','ArrayImage');

AOrderAxes_id=findobj(Array_id,'Tag','ArrayClassAxes');
GOrderAxes_id=findobj(Array_id,'Tag','GeneClassAxes');

VScroller_id=findobj(Array_id,'Tag','VerticalScroll');
HScroller_id=findobj(Array_id,'Tag','HorizontalScroll');
Unselect_id=findobj(Array_id,'Tag','UnselectButton');
UnselectMenu_id=findobj(Array_id,'Tag','UnselectMenu');

SelectedGeneEdit_id=findobj(Array_id,'Tag','SelectedGeneEdit');
SelectedGeneNameEdit_id=findobj(Array_id,'Tag','SelectedGeneNameEdit');
SelectedArrayEdit_id=findobj(Array_id,'Tag','SelectedArrayEdit');
SelectedArrayNameEdit_id=findobj(Array_id,'Tag','SelectedArrayNameEdit');

config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
view_struct=config.view_struct;

data=get(Array_id,'UserData');
dimension=size(data,1);
vectors_count=size(data,2);

if (~isempty(data) || strcmp(action,'open')==1 || strcmp(action,'exit')==1 || strcmp(action,'prepare_axis')==1 || strcmp(action,'info')==1)
	switch action
        case 'info'
            helpdlg(sprintf('\n     MStat %s (%s)\n\n     Tymon Rubel 2003-2023\n\n     Institute of Radioelectronics and Multimedia Technology\n     Warsaw University of Technology',app_version,app_date));
                
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%funkcje obsugi operacji plikowych%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		case 'open'
			set(Array_id,'Pointer','watch');
			pause(0.01);

			[data,gorder,aorder,gnames,anames,gcnames,acnames,glist,fname,path]=load_raw(config.open_path);
			if ~isempty(data)
				mstat_cb('close_dialogs');
				mstat_cb('close_lists');

				stat_struct.pval=NaN*ones(size(data,1),1);
				stat_struct.fdr=NaN*ones(size(data,1),1);
				stat_struct.fc=NaN*ones(size(data,1),1);
				stat_struct.n=ones(size(data,1),1);
				stat_struct.test='None';

				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				restore=cell(1,9);
				restore{1}=fname;
				restore{2}=data;
				restore{3}=anames;
				restore{4}=aorder;
				restore{5}=acnames;
				restore{6}=gcnames;
				restore{7}=gorder;
				restore{8}=gnames;
				restore{9}=glist;
				restore{10}=stat_struct;
				set(findobj(get(Array_id,'Child'),'Label','&Edycja'),'UserData',restore);
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				
				set(Array_id,'UserData',data);
				set(SelectedGeneNameEdit_id,'UserData',gnames);
				set(SelectedArrayNameEdit_id,'UserData',anames);
				set(AOrderAxes_id,'UserData',aorder);
				set(GOrderAxes_id,'UserData',gorder);
				set(findobj(Array_id,'Tag','DatasetPanel'),'UserData',glist);
				set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',1,'UserData',acnames);
				set(findobj(Array_id,'Tag','GeneClassPop'),'Value',1,'UserData',gcnames);
				set(findobj(Array_id,'Tag','Select1D'),'UserData',stat_struct);

				%%%%%%%%%Uaktualnienie view_struct%%%%%%%%%
				view_struct.position_x=1;
				view_struct.max_x=1;
				view_struct.position_y=1;
				view_struct.max_y=1;
				view_struct.selected_gene=0;
				view_struct.selected_array=0;
				config.view_struct=view_struct;

				%%%%%%%%%Zapamietanie katalogu%%%%%%%%%
				config.open_path=path;

				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				%%%%%%%%%Uaktywnienie menu i GUI%%%%%%%%%%%
				mstat_cb('update_missing');
				mstat_cb('do_menu');
				mstat_cb('do_gui');

				%%%%%%%%Przygotowanie i wywietlenie obrazu%%%%%%%%
				mstat_cb('prepare_data');
				mstat_cb('prepare_axis');
				mstat_cb('redraw');

				%%%%%%%%%Zmiana belki tytuowej%%%%%%%%%
				set(Array_id,'Name',['MStat - ' fname]);

				%%%%%%%%%Zmiana funkcji obsugi myszy%%%%%%%%%
				set(Array_id,'WindowButtonMotionFcn','mstat_mf');

			end;
			set(Array_id,'Pointer','arrow');

		case 'save'
			if ~isempty(data)
				name=get(Array_id,'Name');
				name=name(strfind(name,'-')+2:length(name));

				anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');
				aorder=get(findobj(Array_id,'Tag','ArrayClassAxes'),'UserData');
				acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'String');

				gnames=get(findobj(Array_id,'Tag','SelectedGeneNameEdit'),'UserData');
				gorder=get(findobj(Array_id,'Tag','GeneClassAxes'),'UserData');
				gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'String');

				[name,path,format]=uiputfile({'*.txt','Plik tekstowy (*.txt)';'*.arff','Plik Weka (*.arff)'},'Zapisz dane do pliku',[config.open_path name]);
				if name~=0
					if ~isempty(strfind(name,'.txt'))	
						name=name(1:length(name)-4);
					end;
					if ~isempty(strfind(name,'.arff'))	
						name=name(1:length(name)-5);
					end;
					filename=[path name];
						
					dataset.data=data;
					dataset.name=name;
					dataset.column_names=anames;
					dataset.row_names=gnames;
					dataset.column_classes=aorder;
					dataset.class_names=acnames;

					set(Array_id,'Pointer','watch');
					pause(0.01);

					%zapisanie danych, nazw probek, nazw genow, podzialu probek na klasy i nazw klas
					if format==1
						save_table(dataset,[filename '.txt'],'%0.7g');
                        %save_table(dataset,[filename '.txt'],'%0.4f');

						%zapisanie podzialu genow na grupy i nazw grup
						if max(gorder)>1
							order=cell(length(gorder),1);
							for i=1:length(order)
								l=cell(1,2);
								l{1}=gnames{i};
								l{2}=num2str(gorder(i));
								order{i}=l;
							end;
							save_cell(order,[filename '.gcl']);
							save_cell(gcnames,[filename '.gcn']);
						end;
					else
						save_arff(dataset,filename);
					end;
					set(Array_id,'Name',['MStat - ' name]);
					set(Array_id,'Pointer','arrow');

					%%%%%%%%%Zapamietanie katalogu%%%%%%%%%
					config.open_path=path;
					set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
				end;
			end;
		
		case 'exit'
			mstat_cb('close_dialogs');
			mstat_cb('close_lists');
			mstat_cb('close_plots');
			
			config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
			for i=1:length(config.ext_path)
				rmpath(config.ext_path{i});
			end;
			for i=1:length(config.java_path)
				javarmpath(config.java_path{i});
			end;
	%		delete(Array_id);
		
		case 'close_dialogs'
			delete(findobj(0,'Tag','ACluster'));
			delete(findobj(0,'Tag','GCluster'));
			delete(findobj(0,'Tag','GA'));
			delete(findobj(0,'Tag','Adjust'));
			delete(findobj(0,'Tag','ANorm'));
			delete(findobj(0,'Tag','GNorm'));
			delete(findobj(0,'Tag','Missing'));
			delete(findobj(0,'Tag','FilterGenes'));
			delete(findobj(0,'Tag','FilterArrays'));
			delete(findobj(0,'Tag','FilterClasses'));
			delete(findobj(0,'Tag','Selection'));
			delete(findobj(0,'Tag','Selection1D'));
			delete(findobj(0,'Tag','Selection1D_corr'));
			delete(findobj(0,'Tag','Selection1D_svd'));
			delete(findobj(0,'Tag','Selection1D_ica'));
			delete(findobj(0,'Tag','Denoise'));
			delete(findobj(0,'Tag','SVD'));
			delete(findobj(0,'Tag','ICA'));
			delete(findobj(0,'Tag','Test'));
			delete(findobj(0,'Tag','Test1D'));
			delete(findobj(0,'Tag','QC'));
			delete(findobj(0,'Tag','Rollup'));
			
		case 'close_lists'
			delete(findobj(0,'Tag','ValueTable'));
			delete(findobj(0,'Tag','GeneTable'));
			delete(findobj(0,'Tag','ArrayTable'));
			delete(findobj(0,'Tag','ClassTable'));
			delete(findobj(0,'Tag','GroupTable'));
			delete(findobj(0,'Tag','MissingTable'));
			
		case 'close_plots'
			delete(findobj(0,'Tag','PlotPVal'));
			delete(findobj(0,'Tag','PlotPValHist'));
			delete(findobj(0,'Tag','PlotCoeffs'));
			delete(findobj(0,'Tag','PlotCorr'));
			delete(findobj(0,'Tag','PlotProfile'));
			delete(findobj(0,'Tag','PlotProfiles'));
			delete(findobj(0,'Tag','PlotCentroids'));
			delete(findobj(0,'Tag','PlotHistogram'));
			delete(findobj(0,'Tag','PlotAHierClust'));
			delete(findobj(0,'Tag','PlotGHierClust'));
			delete(findobj(0,'Tag','PlotPCA'));
			delete(findobj(0,'Tag','PlotPCA3D'));
			delete(findobj(0,'Tag','PlotSammon'));
			delete(findobj(0,'Tag','PlotSilhouette'));
			delete(findobj(0,'Tag','PlotBox'));
			delete(findobj(0,'Tag','PlotCV'));
			delete(findobj(0,'Tag','PlotEigen'));
			delete(findobj(0,'Tag','PlotScatter'));

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%funkcje zwizane z wywietlaniem obrazu%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		case 'prepare_data'
			%image_data=zeros(dimension,vectors_count);

			min_value=min(data(:));
			max_value=max(data(:));
			
			missing_info=get(findobj(Array_id,'Tag','ValueEdit'),'UserData');
			if missing_info(1)~=0
				missing_list=find(isnan(data));
			end;
			
			absolute=get(findobj(Array_id,'Tag','ViewModeSubMenu'),'UserData');
			if absolute==0
				m=nan_mean(data,2);
				nan_list=isnan(m);
				m(nan_list)=0;
				
				data=data-repmat(m,1,vectors_count);
				
				sd=nan_iqr(data,2)/1.349;
				zeros_list=find(sd<2*eps);
				sd(zeros_list)=1;
				
				image_data=data./repmat(sd,1,vectors_count);
				image_data(zeros_list,:)=0;
				
				range=view_struct.view_relative_range;
				%image_data(find(image_data<-1*range))=-1*range;
				%image_data(find(image_data>range))=range;
                image_data((image_data<-1*range))=-1*range;
				image_data((image_data>range))=range;
				image_data=ceil(254*((image_data+range)/(2*range)))+1;
			else
				image_data=data;
				if min_value>0
					image_data=ceil(254*((image_data-min_value)/(max_value-min_value)))+1;
				else
					range=0.8*max(max_value,abs(min_value));
					image_data(find(image_data<-1*range))=-1*range;
					image_data(find(image_data>range))=range;
					image_data=ceil(254*((image_data+range)/(2*range)))+1;
				end;
			end;

			if missing_info(1)~=0
				image_data(missing_list)=Inf;
			end;

			set(ArrayAxes_id,'UserData',image_data);
		
		case 'prepare_axis'
			%stale
			plot_axis_size=[360 412];
			plot_axis_pos=[25 30];
			order_axis_size=15;
			scroll_size=19;

			fig_pos=get(Array_id,'Position');

			if (~isempty(data))
				%rozmiary obrazka
				info_panel=findobj(Array_id,'Tag','InfoPanel');
				info_panel_pos=get(info_panel,'Position');

				max_size_x=fig_pos(3)-info_panel_pos(3)-plot_axis_pos(1)-scroll_size-10-10-10;
				max_size_x=max_size_x-mod(max_size_x,view_struct.zoom);
				size_x=min(vectors_count*view_struct.zoom,max_size_x);
				view_struct.max_x=min(vectors_count,size_x/view_struct.zoom)-1;
				
				max_size_y=fig_pos(4)-plot_axis_pos(2)-scroll_size-32;
				max_size_y=max_size_y-mod(max_size_y,view_struct.zoom);
				size_y=min(dimension*view_struct.zoom,max_size_y);
				view_struct.max_y=min(dimension,size_y/view_struct.zoom)-1;

				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				plot_axis_size=[size_x size_y];
			end;

			%panel rysunku
			plot_panel=findobj(Array_id,'Tag','PlotPanel');
			plot_panel_size=[plot_axis_size(1)+plot_axis_pos(1)+scroll_size+10 plot_axis_size(2)+plot_axis_pos(2)+scroll_size+20];
			plot_panel_pos=[10 fig_pos(4)-plot_panel_size(2)-6];
			set(plot_panel,'Position',[plot_panel_pos(1) plot_panel_pos(2) plot_panel_size(1) plot_panel_size(2)]);
			
			%rysunek
			set(ArrayAxes_id,'Position',[plot_axis_pos(1) plot_axis_pos(2) plot_axis_size(1) plot_axis_size(2)],'XLim',[0.5 plot_axis_size(1)+0.5],'YLim',[0.5 plot_axis_size(2)+0.5]);
			set(ArrayImage_id,'XData',[1 plot_axis_size(1)],'YData',[1 plot_axis_size(2)]);
			
			%przypisanie  
			set(AOrderAxes_id,'Position',[plot_axis_pos(1) plot_axis_pos(2)+plot_axis_size(2)+5 plot_axis_size(1) order_axis_size],'XLim',[0 view_struct.max_x+1],'YLim',[0 1]);
			set(GOrderAxes_id,'Position',[5 plot_axis_pos(2) order_axis_size plot_axis_size(2)],'XLim',[0 1],'YLim',[0 view_struct.max_y+1]);
	
			%suwaki
			set(VScroller_id,'Position',[plot_axis_pos(1)+plot_axis_size(1) plot_axis_pos(2) scroll_size plot_axis_size(2)]);
			if (~isempty(data) && dimension*view_struct.zoom>max_size_y)
				set(VScroller_id,'Enable','on','Min',0,'Max',1,'SliderStep',[1/(dimension-view_struct.max_y) 10/(dimension-view_struct.max_y)]);
				set(VScroller_id,'Value',min(max(1-(view_struct.position_y-1)/(dimension-view_struct.max_y),0),1));
			else
				set(VScroller_id,'Enable','off');
			end;

			set(HScroller_id,'Position',[plot_axis_pos(1) plot_axis_pos(2)-scroll_size-1 plot_axis_size(1) scroll_size]);
			if (~isempty(data) && vectors_count*view_struct.zoom>max_size_x)
				set(HScroller_id,'Enable','on','Min',0,'Max',1,'SliderStep',[1/(vectors_count-view_struct.max_x) 10/(vectors_count-view_struct.max_x)]);
				set(HScroller_id,'Value',min(max((view_struct.position_x-1)/(vectors_count-view_struct.max_x),0),1));
			else
				set(HScroller_id,'Enable','off');
			end;
			
			set(Unselect_id,'Position',[plot_axis_pos(1)+plot_axis_size(1) plot_axis_pos(2)-scroll_size-1 scroll_size scroll_size]);

		case 'redraw'
			if (~isempty(data) && view_struct.lock~=1)
				view_struct.lock=1;
				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
				
				colors_count=size(view_struct.ctable,1);
				image_data=get(ArrayAxes_id,'UserData');

				max_x=view_struct.max_x;
				max_y=view_struct.max_y;

				position_x=min(view_struct.position_x,vectors_count-max_x);
				position_y=min(view_struct.position_y,dimension-max_y);

				image=im_scale(image_data(position_y:position_y+max_y,position_x:position_x+max_x),view_struct.zoom);
				set(ArrayImage_id,'CData',image);
	%			pause(0.01);

	%			im=zeros(size(image,1),size(image,2),3);
	%			r=config.view_struct.cmap(:,1);
	%			im(:,:,1)=r(image);
	%			r=config.view_struct.cmap(:,2);
	%			im(:,:,2)=r(image);
	%			r=config.view_struct.cmap(:,3);
	%			im(:,:,3)=r(image);
	%			set(ArrayImage_id,'CData',im);
	%			pause(0.01);

				%zamienic na wyswietlanie gotowego obrazu
				aorder=get(AOrderAxes_id,'UserData');
				delete(get(AOrderAxes_id,'Children'));
				axes(AOrderAxes_id);
				hold on;
				hnd=fill([0:max_x;0:max_x;1:max_x+1;1:max_x+1],repmat([0;1;1;0],1,max_x+1),'g');
				if view_struct.zoom==1
					for i=0:max_x
						cindex=mod(aorder(position_x+i),colors_count);
						if cindex==0
							cindex=colors_count;
						end;
						set(hnd(i+1),'FaceColor',view_struct.ctable(cindex,:),'EdgeColor',view_struct.ctable(cindex,:));
					end;
				else
					for i=0:max_x
						cindex=mod(aorder(position_x+i),colors_count);
						if cindex==0
							cindex=colors_count;
						end;
						set(hnd(i+1),'FaceColor',view_struct.ctable(cindex,:));
					end;
				end;
				hold off;
				pause(0.001);

				gorder=get(GOrderAxes_id,'UserData');
				delete(get(GOrderAxes_id,'Children'));
				axes(GOrderAxes_id);

				hold on;
				hnd=fill(repmat([0;1;1;0],1,max_y+1),[0:max_y;0:max_y;1:max_y+1;1:max_y+1],view_struct.ctable(1,:));
				if max(gorder>1)
					if view_struct.zoom==1
						for i=0:max_y
							cindex=mod(gorder(position_y+i),colors_count);
							if cindex==0
								cindex=colors_count;
							end;
							set(hnd(i+1),'FaceColor',view_struct.ctable(cindex,:),'EdgeColor',view_struct.ctable(cindex,:));
						end;
					else
						for i=0:max_y
							cindex=mod(gorder(position_y+i),colors_count);
							if cindex==0
								cindex=colors_count;
							end;
							set(hnd(i+1),'FaceColor',view_struct.ctable(cindex,:));
						end;
					end;
				else
					if view_struct.zoom==1
						set(hnd,'EdgeColor',view_struct.ctable(1,:));
					end;
				end;
				hold off;
				axes(ArrayAxes_id);
				pause(0.001);

				view_struct.position_x=position_x;
				view_struct.position_y=position_y;
				
				AxesPosition=get(ArrayAxes_id,'Position');
				delete(findobj(ArrayAxes_id,'Type','rectangle'));

				gene_nr=view_struct.selected_gene;
				gene_segment=view_struct.position_y;
				if gene_nr>=gene_segment && gene_nr<=gene_segment+view_struct.max_y
					gene_offset=view_struct.zoom*(gene_nr-gene_segment);
					rectangle('Position',[0 gene_offset AxesPosition(3) view_struct.zoom],'HitTest','off','Clipping','off','EdgeColor',[1 1 0]);
				end;
				
				array_nr=view_struct.selected_array;
				array_segment=view_struct.position_x;
				if array_nr>=array_segment && array_nr<=array_segment+view_struct.max_x
					array_offset=view_struct.zoom*(array_nr-array_segment);
					rectangle('Position',[array_offset 0 view_struct.zoom AxesPosition(4)],'HitTest','off','Clipping','off','EdgeColor',[1 1 0]);
				end;
				%pause(0.01);

				set(findobj(Array_id,'Tag','ArrayRangeEdit1'),'String',num2str(view_struct.position_x));
				set(findobj(Array_id,'Tag','ArrayRangeEdit2'),'String',num2str(view_struct.position_x+view_struct.max_x));
				set(findobj(Array_id,'Tag','GeneRangeEdit1'),'String',num2str(view_struct.position_y));
				set(findobj(Array_id,'Tag','GeneRangeEdit2'),'String',num2str(view_struct.position_y+view_struct.max_y));
				%pause(0.01);

				drawnow;

				view_struct.lock=0;
				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
			end;
			
		case 'scroll'
			if view_struct.lock~=1
				view_struct.position_x=max(1,round((vectors_count-view_struct.max_x)*get(HScroller_id,'Value')));
				view_struct.position_y=max(1,round((dimension-view_struct.max_y)*(1-get(VScroller_id,'Value'))));

				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				mstat_cb('redraw');
			end;
		
		case 'zoom_in'
			view_struct.zoom=min(2*view_struct.zoom,view_struct.zoom_range(2));
			config.view_struct=view_struct;
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

			mstat_cb('prepare_axis');
			mstat_cb('do_menu');
			mstat_cb('redraw');

		case 'zoom_out'
			view_struct.zoom=max(0.5*view_struct.zoom,view_struct.zoom_range(1));
			config.view_struct=view_struct;
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

			mstat_cb('prepare_axis');
			mstat_cb('do_menu');
			mstat_cb('redraw');

		case 'click'
	%		 You will get 'normal' for left click and 'alt' for right (or mid) click. 
			if ~strcmp(get(gcbf,'SelectionType'),'alt')
				mstat_cb('mark_row');
			end;
		
		case 'mark_row'
			if gcbo==ArrayImage_id
				AxesPosition=get(ArrayAxes_id,'Position');
				PanelPosition=get(findobj(Array_id,'Tag','PlotPanel'),'Position');
				
				CurrentPoint=get(Array_id,'CurrentPoint');
				
				x=CurrentPoint(1,1);
				y=CurrentPoint(1,2);
			
				pos_x=x-AxesPosition(1)-PanelPosition(1);
				pos_y=AxesPosition(4)+AxesPosition(2)+PanelPosition(2)-y;

				array_segment=view_struct.position_x;
				gene_segment=view_struct.position_y;

				array_offset=floor(pos_x/view_struct.zoom);
				gene_offset=floor(pos_y/view_struct.zoom);
				array_nr=array_segment+array_offset;
				gene_nr=gene_segment+gene_offset;
			else
				AxesPosition=get(AOrderAxes_id,'Position');
				CurrentPoint=get(Array_id,'CurrentPoint');
			
				x=CurrentPoint(1,1);
				pos_x=x-AxesPosition(1);
				array_segment=view_struct.position_x;
				array_offset=floor(pos_x/view_struct.zoom);
				array_nr=array_segment+array_offset;
				gene_nr=1;
			end;
			
			if array_nr>=1 && array_nr<=vectors_count && gene_nr>=1 && gene_nr<=dimension
				view_struct.selected_gene=gene_nr;
				view_struct.selected_array=array_nr;

				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
				mstat_cb('do_gui');
				mstat_cb('redraw_selection');
			end;

		case 'reset_view'
			view_struct.selected_gene=0;
			view_struct.selected_array=0;

			view_struct.position_x=1;
			view_struct.position_y=1;
			config.view_struct=view_struct;
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

		case 'reset_selection'
			view_struct.selected_gene=0;
			view_struct.selected_array=0;

			config.view_struct=view_struct;
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
			
		case 'redraw_selection'
			axes(ArrayAxes_id);
			AxesPosition=get(ArrayAxes_id,'Position');
			delete(findobj(ArrayAxes_id,'Type','rectangle'));

			gene_nr=view_struct.selected_gene;
			gene_segment=view_struct.position_y;
			if gene_nr>=gene_segment && gene_nr<=gene_segment+view_struct.max_y
				gene_offset=view_struct.zoom*(gene_nr-gene_segment);
				rectangle('Position',[0 gene_offset AxesPosition(3) view_struct.zoom],'HitTest','off','Clipping','off','EdgeColor',[1 1 0]);
			end;
			
			array_nr=view_struct.selected_array;
			array_segment=view_struct.position_x;
			if array_nr>=array_segment && array_nr<=array_segment+view_struct.max_x
				array_offset=view_struct.zoom*(array_nr-array_segment);
				rectangle('Position',[array_offset 0 view_struct.zoom AxesPosition(4)],'HitTest','off','Clipping','off','EdgeColor',[1 1 0]);
			end;
			drawnow;

		case 'do_menu'
			FileMenu_id=findobj(Array_id,'Label','&Plik');
			FileSave_id=findobj(FileMenu_id,'Label','Zapi&sz');
			FileImportMenu_id=findobj(FileMenu_id,'Label','Importuj');
			FileExportMenu_id=findobj(FileMenu_id,'Label','Eksportuj');
			FileExportRaportItem_id=findobj(FileExportMenu_id,'Label','&Raport');

			EditMenu_id=findobj(Array_id,'Label','&Edycja');

			ViewMenu_id=findobj(Array_id,'Label','&Widok');
			ViewZoomInMenu_id=findobj(ViewMenu_id,'Label','&Powikszenie');
			ViewZoomOutMenu_id=findobj(ViewMenu_id,'Label','&Zmniejszenie');

			DataMenu_id=findobj(Array_id,'Label','&Dane');

			InfoMenu_id=findobj(Array_id,'Label','&Informacje');
			InfoProfileItem_id=findobj(InfoMenu_id,'Label','&Profil ekspresji');
			InfoBoxplotItem_id=findobj(InfoMenu_id,'Label','&Box plot');						
			InfoHistItem_id=findobj(InfoMenu_id,'Label','&Histogram');
			InfoTestsItem_id=findobj(InfoMenu_id,'Label','&Testy istotnoci');
			InfoScatterplotItem_id=findobj(InfoMenu_id,'Label','&Scatterplot');
			InfoCVplotItem_id=findobj(InfoMenu_id,'Label','&CV plot');
						
			ProcessMenu_id=findobj(Array_id,'Label','&Obrbka');
			ProcessMissingItem_id=findobj(ProcessMenu_id,'Label','&Brakujce wartoci');
			ProcessAdjustItem_id=findobj(ProcessMenu_id,'Label','&Zmiana zakresu');
			ProcessFilterItem_id=findobj(ProcessMenu_id,'Label','&Filtracja');
			ProcessRollupItem_id=findobj(ProcessMenu_id,'Label','&Urednianie');
			ProcessSelectItem_id=findobj(ProcessMenu_id,'Label','&Selekcja');
			ProcessNormalizeSubmenu_id=findobj(ProcessMenu_id,'Label','&Normalizacja');
			ProcessNormalizeGenesItem_id=findobj(ProcessNormalizeSubmenu_id,'Label','&Cech');
			
			ToolbarFileSave_id=findobj(Array_id,'Tag','ToolbarFileSave');
			ToolbarUndo_id=findobj(Array_id,'Tag','ToolbarUndo');
			ToolbarSaveState_id=findobj(Array_id,'Tag','ToolbarSaveState');
			ToolbarZoomIn_id=findobj(Array_id,'Tag','ToolbarZoomIn');
			ToolbarZoomOut_id=findobj(Array_id,'Tag','ToolbarZoomOut');
			
			aorder=get(AOrderAxes_id,'UserData');
			missing_info=get(findobj(Array_id,'Tag','ValueEdit'),'UserData');

			if ~isempty(data)
				set(FileSave_id,'Enable','on');
				set(ToolbarFileSave_id,'CData',imread( fullfile(config.ui_images_path,'save.png'),'BackgroundColor',config.bg_color));
				set(ToolbarSaveState_id,'CData',imread( fullfile(config.ui_images_path,'redo.png'),'BackgroundColor',config.bg_color));
				set(ToolbarUndo_id,'CData',imread( fullfile(config.ui_images_path,'undo.png'),'BackgroundColor',config.bg_color));
				if view_struct.zoom==view_struct.zoom_range(1)
					set(ToolbarZoomOut_id,'CData',imread( fullfile(config.ui_images_path,'zoomout_g.png'),'BackgroundColor',config.bg_color));
					set(ViewZoomOutMenu_id,'Enable','off');
				else
					set(ToolbarZoomOut_id,'CData',imread( fullfile(config.ui_images_path,'zoomout.png'),'BackgroundColor',config.bg_color));
					set(ViewZoomOutMenu_id,'Enable','on')
					if view_struct.zoom==view_struct.zoom_range(2)
						set(ToolbarZoomIn_id,'CData',imread( fullfile(config.ui_images_path,'zoomin_g.png'),'BackgroundColor',config.bg_color));
						set(ViewZoomInMenu_id,'Enable','off');
					else
						set(ToolbarZoomIn_id,'CData',imread( fullfile(config.ui_images_path,'zoomin.png'),'BackgroundColor',config.bg_color));
						set(ViewZoomInMenu_id,'Enable','on');
					end
				end;

				set(FileImportMenu_id,'Enable','on');
				set(FileExportMenu_id,'Enable','on');
				set(EditMenu_id,'Enable','on');
				set(ViewMenu_id,'Enable','on');
				set(DataMenu_id,'Enable','on');
				set(InfoMenu_id,'Enable','on');
				set(ProcessMenu_id,'Enable','on');
				
				if missing_info(1)==0
					set(FileExportRaportItem_id,'Enable','on');

					set(get(InfoMenu_id,'Child'),'Enable','on');					
					
					set(get(ProcessMenu_id,'Child'),'Enable','on');
					set(ProcessMissingItem_id,'Enable','off');
					set(ProcessNormalizeGenesItem_id,'Enable','on');
					if max(aorder)>1
						set(findobj(get(ProcessSelectItem_id,'Child'),'Tag','Select1D'),'Enable','on');
						set(findobj(get(ProcessSelectItem_id,'Child'),'Tag','SelectND'),'Enable','on');
					else
						set(findobj(get(ProcessSelectItem_id,'Child'),'Tag','Select1D'),'Enable','off');
						set(findobj(get(ProcessSelectItem_id,'Child'),'Tag','SelectND'),'Enable','off');
					end;
				else
					set(FileExportRaportItem_id,'Enable','off');
					
					set(get(InfoMenu_id,'Child'),'Enable','off');
					set(InfoProfileItem_id,'Enable','on');
					set(InfoBoxplotItem_id,'Enable','on');					
					set(InfoHistItem_id,'Enable','on');
					set(InfoScatterplotItem_id,'Enable','on');
					set(InfoCVplotItem_id,'Enable','on');
					
					set(get(ProcessMenu_id,'Child'),'Enable','off');
					set(ProcessMissingItem_id,'Enable','on');
					set(ProcessFilterItem_id,'Enable','on');
					set(ProcessAdjustItem_id,'Enable','on');
					set(ProcessRollupItem_id,'Enable','on');
					set(ProcessMissingItem_id,'Enable','on');
					set(ProcessSelectItem_id,'Enable','on');
					set(ProcessNormalizeSubmenu_id,'Enable','on');
					set(ProcessNormalizeGenesItem_id,'Enable','off');
				end;
				
				if max(aorder)>1
					set(InfoTestsItem_id,'Enable','on');
				else
					set(InfoTestsItem_id,'Enable','off');
				end;

			else
				set(FileSave_id,'Enable','off');
				set(ToolbarFileSave_id,'CData',imread( fullfile(config.ui_images_path,'save_g.png'),'BackgroundColor',config.bg_color));
				set(ToolbarSaveState_id,'CData',imread( fullfile(config.ui_images_path,'redo_g.png'),'BackgroundColor',config.bg_color));
				set(ToolbarUndo_id,'CData',imread( fullfile(config.ui_images_path,'undo_g.png'),'BackgroundColor',config.bg_color));
				set(ToolbarZoomIn_id,'CData',imread( fullfile(config.ui_images_path,'zoomin_g.png'),'BackgroundColor',config.bg_color));
				set(ToolbarZoomOut_id,'CData',imread( fullfile(config.ui_images_path,'zoomout_g.png'),'BackgroundColor',config.bg_color));
				set(FileImportMenu_id,'Enable','off');
				set(FileExportMenu_id,'Enable','off');
				set(EditMenu_id,'Enable','off');
				set(ViewMenu_id,'Enable','off');
				set(ProcessMenu_id,'Enable','off');
			end;

		case 'do_gui'
			if view_struct.lock~=1
				view_struct.lock=1;
				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				aorder=get(AOrderAxes_id,'UserData');
				gorder=get(GOrderAxes_id,'UserData');

				anames=get(SelectedArrayNameEdit_id,'UserData');
				gnames=get(SelectedGeneNameEdit_id,'UserData');

				acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'UserData');
				gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'UserData');

				sel_array=view_struct.selected_array;
				sel_gene=view_struct.selected_gene;

				PopupMenu_id=get(ArrayImage_id,'UIContextMenu');

				if ~isempty(data)
					set(findobj(Array_id,'Tag','ArrayNrEdit'),'String',num2str(vectors_count));
					set(findobj(Array_id,'Tag','GeneNrEdit'),'String',num2str(dimension));
					mstat_cb('do_gui_view_range');

					if size(acnames,1)>1
						set(findobj(Array_id,'Tag','ArrayClassPop'),'Enable','on','String',acnames);
						set(findobj(Array_id,'Tag','ArrayReorderButt'),'Enable','on');
					else
						set(findobj(Array_id,'Tag','ArrayClassPop'),'Enable','off','String',acnames);
						set(findobj(Array_id,'Tag','ArrayReorderButt'),'Enable','off');
					end;

					if size(gcnames,1)>1
						set(findobj(Array_id,'Tag','GeneClassPop'),'Enable','on','String',gcnames);
						set(findobj(Array_id,'Tag','GeneReorderButt'),'Enable','on');
					else
						set(findobj(Array_id,'Tag','GeneClassPop'),'Enable','off','String',gcnames);
						set(findobj(Array_id,'Tag','GeneReorderButt'),'Enable','off');
					end;

					if sel_gene~=0
						set(SelectedArrayEdit_id,'String',num2str(sel_array),'UserData',sel_array);
						set(SelectedArrayNameEdit_id,'String',anames{sel_array});
						set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',aorder(sel_array));

						set(SelectedGeneEdit_id,'String',num2str(sel_gene),'UserData',sel_gene);
						set(SelectedGeneNameEdit_id,'String',gnames{sel_gene});
						set(findobj(Array_id,'Tag','GeneClassPop'),'Value',gorder(sel_gene));
						set(findobj(Array_id,'Tag','ValueEdit'),'String',num2str(data(sel_gene,sel_array),'%0.5f'));
						set(findobj(Array_id,'Tag','ArrayDeleteButt'),'Enable','on');
						set(findobj(Array_id,'Tag','ArrayClassAssignButt'),'Enable','on');
						set(findobj(Array_id,'Tag','ArrayClassJoinButt'),'Enable','on');
						set(findobj(Array_id,'Tag','ArrayClassDeleteButt'),'Enable','on');
						set(findobj(Array_id,'Tag','GeneDeleteButt'),'Enable','on');
						set(findobj(Array_id,'Tag','GeneClassAssignButt'),'Enable','on');
						set(findobj(Array_id,'Tag','GeneClassJoinButt'),'Enable','on');
						set(findobj(Array_id,'Tag','GeneClassDeleteButt'),'Enable','on');

						set(get(PopupMenu_id,'Child'),'Enable','on');

						gin=get(findobj(Array_id,'Tag','GeneInfo'),'UserData');
						if ~isempty(gin)
							set(findobj(Array_id,'Tag','GeneInfo'),'Enable','on');
							set(findobj(Array_id,'Tag','GeneInfoPopup'),'Enable','on');
						else
							set(findobj(Array_id,'Tag','GeneInfo'),'Enable','off');
							set(findobj(Array_id,'Tag','GeneInfoPopup'),'Enable','off');
						end;

						set(Unselect_id,'Enable','on');
						set(UnselectMenu_id,'Enable','on');

						if ~isempty(findobj('Tag','GeneTable'))
							genes_table('set_selection');
						end;
						if ~isempty(findobj('Tag','ArrayTable'))
							arrays_table('set_selection');
						end;
						if ~isempty(findobj('Tag','GroupTable'))
							groups_table('set_selection');
						end;
						if ~isempty(findobj('Tag','ClassTable'))
							classes_table('set_selection');
						end;
						if ~isempty(findobj('Tag','ValueTable'))
							values_table('set_selection');
						end;						
					else
						set(SelectedArrayEdit_id,'String','','UserData',0);
						set(SelectedArrayNameEdit_id,'String','');
						set(SelectedGeneEdit_id,'String','','UserData',0);
						set(SelectedGeneNameEdit_id,'String','');
						set(findobj(Array_id,'Tag','ValueEdit'),'String','');

						set(findobj(Array_id,'Tag','ArrayDeleteButt'),'Enable','off');
						set(findobj(Array_id,'Tag','ArrayClassAssignButt'),'Enable','off');
						set(findobj(Array_id,'Tag','ArrayClassJoinButt'),'Enable','off');
						set(findobj(Array_id,'Tag','ArrayClassDeleteButt'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneDeleteButt'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneClassAssignButt'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneClassJoinButt'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneClassDeleteButt'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneGroupProfilesButt'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneProfileButt'),'Enable','off');

						set(findobj(Array_id,'Tag','GeneInfo'),'Enable','off');
						set(findobj(Array_id,'Tag','GeneInfoPopup'),'Enable','off');

						set(get(PopupMenu_id,'Child'),'Enable','off');

						set(Unselect_id,'Enable','off');
						set(UnselectMenu_id,'Enable','off');
					end;
				else
					set(findobj(Array_id,'Tag','GeneCentroidsButt'),'Enable','off');
				end;
				view_struct.lock=0;
				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
			end;
			
		case 'do_gui_view_range'
			set(findobj(Array_id,'Tag','ArrayRangeEdit1'),'String',num2str(view_struct.position_x));
			set(findobj(Array_id,'Tag','ArrayRangeEdit2'),'String',num2str(view_struct.position_x+view_struct.max_x));
			set(findobj(Array_id,'Tag','GeneRangeEdit1'),'String',num2str(view_struct.position_y));
			set(findobj(Array_id,'Tag','GeneRangeEdit2'),'String',num2str(view_struct.position_y+view_struct.max_y));


		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%                              %
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

		case 'update_view'
			mstat_cb('prepare_data');
			mstat_cb('redraw');

		case 'unselect'
			mstat_cb('reset_selection');
			mstat_cb('redraw');
			mstat_cb('do_gui');

		case 'update_missing'
			
	%		mean(-1*sum(isnan(data)-1))
			missing_info=zeros(1,2);
			[missing_info(1),missing_info(2)]=missing_percent(data);
			missing_info(2)=size(data,1)-missing_info(2);
			set(findobj(Array_id,'Tag','ValueEdit'),'UserData',missing_info);

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%Obsuga kontrolek w ramce Cechy%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		case 'gene_info'
			gin=get(findobj(Array_id,'Tag','GeneInfo'),'UserData');
			if ~isempty(gin)
				gnames=get(SelectedGeneNameEdit_id,'UserData');
				selected_nr=view_struct.selected_gene;
				if selected_nr>0
					str=gnames{selected_nr};
					gene_nr=find_str(str,gin.ProbeSetName);
					if gene_nr>0
						str=gin.Description{gene_nr};
					end;
				end;
			end;

		case 'goto_gene'
			Edit_id=gcbo;
			num=str2num(get(Edit_id,'String'));
			if ~isempty(num)
				view_struct.position_y=max(1,min(num,dimension-view_struct.max_y));

				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				set(VScroller_id,'Value',min(max(1-(view_struct.position_y-1)/(dimension-view_struct.max_y),0),1));
				mstat_cb('redraw');
			end;
			mstat_cb('do_gui');

		case 'select_gene'
			num=str2num(get(SelectedGeneEdit_id,'String'));
			if isempty(num) || num<1 || num>dimension
				set(SelectedGeneEdit_id,'String',num2str(view_struct.selected_gene));
				beep;
				return;
			else
				view_struct.selected_gene=num;
				view_struct.selected_array=max(view_struct.selected_array,1);
				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
%
				mstat_cb('set_selected_gene');
			end;

		case 'set_selected_gene'
			redraw=0;
			if view_struct.selected_gene>view_struct.position_y+view_struct.max_y || view_struct.selected_gene<view_struct.position_y
				view_struct.position_y=max(1,min(view_struct.selected_gene,dimension-view_struct.max_y));
				redraw=1;
			end;

			config.view_struct=view_struct;
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

			mstat_cb('do_gui');
			if redraw==1
				mstat_cb('redraw');
				set(VScroller_id,'Value',min(max(1-(view_struct.position_y-1)/(dimension-view_struct.max_y),0),1));
			else
				mstat_cb('redraw_selection');
			end;

		case 'find_gene'
			selected_nr=view_struct.selected_gene;
			if selected_nr>0
				selected_nr=selected_nr+1;
			else
				selected_nr=1;
			end;

			[str,exact,to_parentesis,back]=finddlg('Szukaj cechy',config.gstr);
			pause(0.1);
			if ~isempty(str)
				str=strtrim(char(str));
				gnames=get(SelectedGeneNameEdit_id,'UserData');
				gene_nr=find_str(str,gnames,selected_nr,length(gnames),exact,to_parentesis);
				if gene_nr<=0 & selected_nr~=1
					selected_option=questdlg('Czy szuka od pocztku zbioru?','MStat','Tak','Nie','Tak');
					pause(0.1); 
					if strcmp(selected_option,'Tak')
						gene_nr=find_str(str,gnames,1,selected_nr,exact,to_parentesis);
					end;
				end;

				if gene_nr>0
					view_struct.selected_gene=gene_nr;
					view_struct.selected_array=max(view_struct.selected_array,1);
					config.view_struct=view_struct;
					config.gstr=cellstr(str);
					set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

					mstat_cb('set_selected_gene');
				end;
			end;

		case 'delete_gene'
			glist=get(findobj(Array_id,'Tag','DatasetPanel'),'UserData');
			if length(glist)~=1
				
				image_data=get(ArrayAxes_id,'UserData');
				gnames=get(SelectedGeneNameEdit_id,'UserData');
				gorder=get(GOrderAxes_id,'UserData');
				stat_struct=get(findobj(Array_id,'Tag','Select1D'),'UserData');

				class_nr=size(get(findobj(Array_id,'Tag','GeneClassPop'),'String'),1);
				gene_nr=view_struct.selected_gene;

				data(gene_nr,:)=[];
				image_data(gene_nr,:)=[];
				gorder(gene_nr)=[];
				gnames(gene_nr,:)=[];
				glist(gene_nr)=[];
				stat_struct.pval(gene_nr)=[];
				stat_struct.fdr(gene_nr)=[];
				stat_struct.fc(gene_nr)=[];
				stat_struct.n(gene_nr)=[];

				set(Array_id,'UserData',data);
				set(ArrayAxes_id,'UserData',image_data);
				set(SelectedGeneNameEdit_id,'UserData',gnames);
				set(GOrderAxes_id,'UserData',gorder);
				set(findobj(Array_id,'Tag','DatasetPanel'),'UserData',glist);
				set(findobj(Array_id,'Tag','Select1D'),'UserData',stat_struct);

				%%%%%%usunicie grupy jeeli wszystkie cechy zostay usunite%%%%%%
				for i=1:class_nr
					if length(find(gorder==i))==0
						set(findobj(Array_id,'Tag','GeneClassPop'),'Value',i);
						mstat_cb('delete_gclass');
						return;
					end;
				end;

				mstat_cb('reset_view');
				mstat_cb('update_missing');
				mstat_cb('do_menu');
				mstat_cb('do_gui');
				mstat_cb('prepare_axis');
				mstat_cb('redraw');

				%%%%%%aktualizacja tabel%%%%%%
				ValueTable_id=findobj('Tag','ValueTable');
				if ~isempty(ValueTable_id)
					table=get(ValueTable_id,'UserData');
					table_model=table.getModel;
					table_model.removeRow(gene_nr-1);
					drawnow;
					table.updateUI;
				end;

				if ~isempty(findobj('Tag','GeneTable'))
					genes_table('remove',gene_nr);
				end;
				
				%%%%%%aktualizacja rysunkow%%%%%%
				PlotScatter_id=findobj('Tag','PlotScatter');
				if ~isempty(PlotScatter_id)
					plot_scatterplot_cb('update',PlotScatter_id);
				end;
			end;

		case 'delete_gclass'
			gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'String');
			if length(gcnames)~=1
				
				image_data=get(ArrayAxes_id,'UserData');

				gnames=get(SelectedGeneNameEdit_id,'UserData');
				gorder=get(GOrderAxes_id,'UserData');
				glist=get(findobj(Array_id,'Tag','DatasetPanel'),'UserData');
				stat_struct=get(findobj(Array_id,'Tag','Select1D'),'UserData');

				selected_class=get(findobj(Array_id,'Tag','GeneClassPop'),'Value');
				
				index=find(gorder==selected_class);
				data(index,:)=[];
				image_data(index,:)=[];
				gnames(index,:)=[];
				gcnames(selected_class,:)=[];
				glist(index)=[];
				gorder(index)=[];
				for i=1:length(gorder)
					if gorder(i)>selected_class
						gorder(i)=gorder(i)-1;
					end;
				end;
				stat_struct.pval(index)=[];
				stat_struct.fdr(index)=[];
				stat_struct.fc(index)=[];
				stat_struct.n(index)=[];
				
				mstat_cb('reset_view');

				set(Array_id,'UserData',data);
				set(ArrayAxes_id,'UserData',image_data);
				set(SelectedGeneNameEdit_id,'UserData',gnames);
				set(GOrderAxes_id,'UserData',gorder);
				set(findobj(Array_id,'Tag','DatasetPanel'),'UserData',glist);
				set(findobj(Array_id,'Tag','GeneClassPop'),'Value',1,'UserData',gcnames);
				set(findobj(Array_id,'Tag','Select1D'),'UserData',stat_struct);

				mstat_cb('update_missing');
				mstat_cb('do_menu');
				mstat_cb('do_gui');
				mstat_cb('prepare_axis');
				mstat_cb('redraw');
				
				if ~isempty(findobj('Tag','GeneTable'))
					genes_table('create');
				end;
			end;

		case 'gene_reorder'
			image_data=get(ArrayAxes_id,'UserData');
			gnames=get(SelectedGeneNameEdit_id,'UserData');
			gorder=get(GOrderAxes_id,'UserData');
			stat_struct=get(findobj(Array_id,'Tag','Select1D'),'UserData');
			glist=get(findobj(Array_id,'Tag','DatasetPanel'),'UserData');

			class_nr=max(gorder);
			
			list=find(gorder==1);
			new_data=data(list,:);
			new_image_data=image_data(list,:);
			new_gnames=gnames(list,:);
			new_glist=glist(list);
			new_gorder=ones(length(list),1);
			new_stat_struct.test=stat_struct.test;
			new_stat_struct.fc=stat_struct.fc(list);
			new_stat_struct.pval=stat_struct.pval(list);
			new_stat_struct.fdr=stat_struct.fdr(list);
			new_stat_struct.n=stat_struct.n(list);
			
			for i=2:class_nr
				list=find(gorder==i);
				new_data=[new_data ; data(list,:)];
				new_image_data=[new_image_data ; image_data(list,:)];
				new_gnames=[new_gnames ; gnames(list,:)];
				new_gorder=[new_gorder ; i*ones(length(list),1)];
				new_glist=[new_glist ; glist(list)];

				new_stat_struct.fc=[new_stat_struct.fc ; stat_struct.fc(list)];
				new_stat_struct.pval=[new_stat_struct.pval ; stat_struct.pval(list)];
				new_stat_struct.fdr=[new_stat_struct.fdr ; stat_struct.fdr(list)];
				new_stat_struct.n=[new_stat_struct.n ; stat_struct.n(list)];
			end;

			set(Array_id,'UserData',new_data);
			set(ArrayAxes_id,'UserData',new_image_data);
			set(SelectedGeneNameEdit_id,'UserData',new_gnames);
			set(GOrderAxes_id,'UserData',new_gorder);
			set(findobj(Array_id,'Tag','DatasetPanel'),'UserData',new_glist);
			set(findobj(Array_id,'Tag','Select1D'),'UserData',new_stat_struct);

			mstat_cb('reset_view');
			mstat_cb('do_gui');
			mstat_cb('prepare_axis');
			mstat_cb('redraw');

			%%%%%%aktualizacja tabel%%%%%%
			if ~isempty(findobj('Tag','ValueTable'))
				values_table('reorder')
			end;
			if ~isempty(findobj('Tag','GeneTable'))
				genes_table('reorder');
			end;


		case 'assign_gene'
			gorder=get(GOrderAxes_id,'UserData');
			gene_nr=view_struct.selected_gene;
			class_nr=size(get(findobj(Array_id,'Tag','GeneClassPop'),'String'),1);

			gorder(gene_nr)=get(findobj(Array_id,'Tag','GeneClassPop'),'Value');

			set(GOrderAxes_id,'UserData',gorder);
			
			%%%%%%usunicie grupy jeeli wszystkie jej cechy zostay doczone do innych%%%%%%
			for i=1:class_nr
				if length(find(gorder==i))==0
					mstat_cb('delete_gclass');
					return;
				end;
			end;

			mstat_cb('redraw');
			mstat_cb('do_menu');

			%%%%%%aktualizacja tabel%%%%%%
						
		case 'join_gclass'
			selected_gene=view_struct.selected_gene;
			if selected_gene~=0
				gorder=get(GOrderAxes_id,'UserData');
				gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'String');

				selected_gene_class=gorder(selected_gene);
				new_class=get(findobj(Array_id,'Tag','GeneClassPop'),'Value');
				if selected_gene_class~=new_class
					gorder(find(gorder==selected_gene_class))=new_class;
					gcnames{new_class}=[gcnames{new_class} '+' gcnames{selected_gene_class}];

					set(GOrderAxes_id,'UserData',gorder);
					set(findobj(Array_id,'Tag','GeneClassPop'),'String',gcnames);
					set(findobj(Array_id,'Tag','GeneClassPop'),'Value',selected_gene_class);
					mstat_cb('delete_gclass');
				end;
			end;

		case 'set_gene_value'
			if view_struct.selected_gene~=0 && view_struct.selected_array~=0
				num=str2num(get(findobj(Array_id,'Tag','ValueEdit'),'String'));
				if ~isempty(num)
					data(view_struct.selected_gene,view_struct.selected_array)=num;
					set(Array_id,'UserData',data);

					mstat_cb('update_missing');
					mstat_cb('prepare_data');
					mstat_cb('redraw');
					mstat_cb('do_menu');
					mstat_cb('do_gui');
				end;
			end;

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%Obsuga kontrolek w ramce Prbki%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		case 'goto_array'
			Edit_id=gcbo;
			num=str2num(get(Edit_id,'String'));
			if ~isempty(num)
				view_struct.position_x=max(1,min(num,vectors_count-view_struct.max_x));

				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				set(HScroller_id,'Value',min(max((view_struct.position_x-1)/(vectors_count-view_struct.max_x),0),1));
				mstat_cb('redraw');
			end;
			mstat_cb('do_gui');

		case 'select_array'
			num=str2num(get(SelectedArrayEdit_id,'String'));
			if isempty(num) || num<1 || num>vectors_count
				set(SelectedArrayEdit_id,'String',num2str(view_struct.selected_array));
				beep;
				return;
			else
				view_struct.selected_array=num;
				view_struct.selected_gene=max(view_struct.selected_gene,1);
				config.view_struct=view_struct;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

				mstat_cb('set_selected_array');
			end;

		case 'set_selected_array'
			redraw=0;
			if view_struct.selected_array>view_struct.position_x+view_struct.max_x || view_struct.selected_array<view_struct.position_x
				view_struct.position_x=max(1,min(view_struct.selected_array,vectors_count-view_struct.max_x));
				redraw=1;
			end;

			config.view_struct=view_struct;
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);

			mstat_cb('do_gui');
			if redraw==1
				mstat_cb('redraw');
				config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
				view_struct=config.view_struct;
				set(HScroller_id,'Value',min(max((view_struct.position_x-1)/(vectors_count-view_struct.max_x),0),1));
			else
				mstat_cb('redraw_selection');
			end;

		case 'find_array'
			selected_nr=view_struct.selected_array;
			if selected_nr>0
				selected_nr=selected_nr+1;
			else
				selected_nr=1;
			end;

			[str,exact,to_parentesis,back]=finddlg('Szukaj prbki',config.astr,0,0);
			pause(0.1); 			
			if ~isempty(str)
				str=strtrim(char(str));
				anames=get(SelectedArrayNameEdit_id,'UserData');
				array_nr=find_str(str,anames,selected_nr,length(anames),exact);
				if array_nr<=0 && selected_nr~=1
					selected_option=questdlg('Czy szuka od pocztku zbioru?','MStat','Tak','Nie','Tak');
					pause(0.1); 
					if strcmp(selected_option,'Tak')
						array_nr=find_str(str,anames,1,selected_nr,exact);
					end;
				end;

				if array_nr>0
					view_struct.selected_array=array_nr;
					view_struct.selected_gene=max(view_struct.selected_gene,1);
					config.view_struct=view_struct;
					config.astr=cellstr(str);
					set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
					
					mstat_cb('set_selected_array');
				end;
			end;

		case 'delete_array'
			anames=get(SelectedArrayNameEdit_id,'UserData');
			if length(anames)~=1
				
				image_data=get(ArrayAxes_id,'UserData');
				aorder=get(AOrderAxes_id,'UserData');
				class_nr=size(get(findobj(Array_id,'Tag','ArrayClassPop'),'String'),1);
		
				array_nr=view_struct.selected_array;

				data(:,array_nr)=[];
				image_data(:,array_nr)=[];
				aorder(array_nr)=[];
				anames(array_nr,:)=[];

				set(Array_id,'UserData',data);
				set(ArrayAxes_id,'UserData',image_data);
				set(SelectedArrayNameEdit_id,'UserData',anames);
				set(AOrderAxes_id,'UserData',aorder);

				mstat_cb('reset_view');

				%%%%%%usunicie klasy jeeli wszystkie jej prbki zostay usunite%%%%%%
				for i=1:class_nr
					if length(find(aorder==i))==0
						set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',i);
						mstat_cb('delete_aclass');
						return;
					end;
				end;

				%%%%%%aktualizacja tabel%%%%%%
				ValueTable_id=findobj('Tag','ValueTable');
				if ~isempty(ValueTable_id)
					table=get(ValueTable_id,'UserData');
					columnModel=table.getColumnModel;
					table.removeColumn(columnModel.getColumn(array_nr));
					drawnow;
					table.updateUI;
				end;
				if ~isempty(findobj('Tag','ArrayTable'))
					arrays_table('remove',array_nr);
				end;

				mstat_cb('update_missing');
				mstat_cb('do_menu');
				mstat_cb('do_gui');
				mstat_cb('prepare_axis');
				mstat_cb('redraw');
			end;

		case 'delete_aclass'
			acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'String');
			if length(acnames)~=1
				
				image_data=get(ArrayAxes_id,'UserData');

				anames=get(SelectedArrayNameEdit_id,'UserData');
				aorder=get(AOrderAxes_id,'UserData');
				selected_class=get(findobj(Array_id,'Tag','ArrayClassPop'),'Value');

				data(:,find(aorder==selected_class))=[];
				image_data(:,find(aorder==selected_class))=[];
				anames(find(aorder==selected_class),:)=[];
				acnames(selected_class,:)=[];
				aorder(find(aorder==selected_class))=[];
				for i=1:length(aorder)
					if aorder(i)>selected_class
						aorder(i)=aorder(i)-1;
					end;
				end;
		
				set(Array_id,'UserData',data);
				set(ArrayAxes_id,'UserData',image_data);
				set(SelectedArrayNameEdit_id,'UserData',anames);
				set(AOrderAxes_id,'UserData',aorder);
				set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',1,'UserData',acnames);

				mstat_cb('reset_view');
				mstat_cb('update_missing');
				mstat_cb('do_gui');
				mstat_cb('do_menu');
				mstat_cb('prepare_data');
				mstat_cb('prepare_axis');
				mstat_cb('redraw');
				
				if ~isempty(findobj('Tag','ArrayTable'))
					arrays_table('create');
				end;
				
				if ~isempty(findobj('Tag','ClassTable'))
					classes_table('remove',selected_class);
				end;
			end;

		case 'assign_array'
			aorder=get(AOrderAxes_id,'UserData');
			array_nr=view_struct.selected_array;
			class_nr=size(get(findobj(Array_id,'Tag','ArrayClassPop'),'String'),1);

			aorder(array_nr)=get(findobj(Array_id,'Tag','ArrayClassPop'),'Value');

			set(AOrderAxes_id,'UserData',aorder);
			
			%%%%%%usunicie klasy jeeli wszystkie jej prbki zostay doczone do innych%%%%%%
			for i=1:class_nr
				if length(find(aorder==i))==0
					set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',i);
					mstat_cb('delete_aclass');
					return;
				end;
			end;

			mstat_cb('redraw');
			mstat_cb('do_menu');

		case 'join_aclass'
			selected_array=view_struct.selected_array;
			if selected_array~=0
				aorder=get(AOrderAxes_id,'UserData');
				acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'String');

				selected_array_class=aorder(selected_array);
				new_class=get(findobj(Array_id,'Tag','ArrayClassPop'),'Value');
				if selected_array_class~=new_class
					aorder(find(aorder==selected_array_class))=new_class;
					acnames{new_class}=[acnames{new_class} '+' acnames{selected_array_class}];

					set(AOrderAxes_id,'UserData',aorder);
					set(findobj(Array_id,'Tag','ArrayClassPop'),'String',acnames);
					set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',selected_array_class);
					mstat_cb('delete_aclass');
				end;
			end;

		case 'array_reorder'
			
			image_data=get(ArrayAxes_id,'UserData');
			anames=get(SelectedArrayNameEdit_id,'UserData');
			aorder=get(AOrderAxes_id,'UserData');
			class_nr=max(aorder);
			
			list=find(aorder==1);
			new_data=data(:,list);
			new_image_data=image_data(:,list);
			new_anames=anames(list,:);
			new_aorder=ones(size(list));
			for i=2:class_nr
				list=find(aorder==i);
				new_data=[new_data data(:,list)];
				new_image_data=[new_image_data image_data(:,list)];
				new_anames=[new_anames;anames(list,:)];
				new_aorder=[new_aorder;i*ones(length(list),1)];
			end;
			
			set(Array_id,'UserData',new_data);
			set(ArrayAxes_id,'UserData',new_image_data);
			set(SelectedArrayNameEdit_id,'UserData',new_anames);
			set(AOrderAxes_id,'UserData',new_aorder);

			mstat_cb('reset_view');
			mstat_cb('do_gui');
			mstat_cb('prepare_axis');
			mstat_cb('redraw');

			%%%%%%aktualizacja tabel%%%%%%
	%		ValueTable_id=findobj('Tag','ValueTable');
	%		if ~isempty(ValueTable_id)
	%			values_table('reorder')
	%		end;

			ArrayTable_id=findobj('Tag','ArrayTable');
			if ~isempty(ArrayTable_id)
				arrays_table('reorder');
			end;


		%%%%%%%%%%%%%%%%%
		%Menu Informacje%
		%%%%%%%%%%%%%%%%%
		case 'aboxplot'
			anames=get(SelectedArrayNameEdit_id,'UserData');
			array_nr=view_struct.selected_array;

			if array_nr~=0
				data=data(:,array_nr);
				names{1}=anames{array_nr};
				name=names{1};
			else
				names=anames;
				name='wszystkie prbki';
			end;
			plot_box(data,[],names,name);

		case 'gboxplot'
			acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'String');
			aorder=get(AOrderAxes_id,'UserData');
			gnames=get(SelectedGeneNameEdit_id,'UserData');
			gene_nr=view_struct.selected_gene;

			if gene_nr~=0
				data=data(gene_nr,:);
				name=gnames{gene_nr};
			else
				adata=[];
				order=[];
				for i=1:max(aorder)
					index=find(aorder==i);
					adata=[adata,reshape(data(:,index),1,size(data,1)*length(index))];
					order=[order,i*ones(1,size(data,1)*length(index))];
				end;
				data=adata;
				aorder=order;
				name='wszystkie cechy';
			end;
			plot_box(data,aorder,acnames,name);

		case 'cv_plot'
			aorder=get(AOrderAxes_id,'UserData');
			
			array_nr=view_struct.selected_array;
			if array_nr~=0
				data=data(:,find(aorder==aorder(array_nr)));

				acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'String');
				name=acnames{aorder(array_nr)};
			else
				name='wszystkie prbki';
			end;
			set(Array_id,'Pointer','watch');
			pause(0.01);
			plot_cv(data,'box','cv',name);
			set(Array_id,'Pointer','arrow');

		%%%%%%%%%%%%
		%Menu Widok%
		%%%%%%%%%%%%
		case 'set_lut'
			hnd=findobj(Array_id,'Tag','ViewLUTSubMenu');
			set(get(hnd,'Child'),'Checked','off');
			set(gcbo,'Checked','on');

			cmap=lut(get(gcbo,'Tag'),256,256);
			set(Array_id,'Colormap',cmap);
			drawnow;

		case 'invert_lut'
			cmap=get(Array_id,'Colormap');
			cmap(1:255,:)=flipud(cmap(1:255,:));

			set(Array_id,'Colormap',cmap);
			drawnow;

		case 'set_viewmode'
			hnd=findobj(Array_id,'Tag','ViewModeSubMenu');
			set(get(hnd,'Child'),'Checked','off');
			set(gcbo,'Checked','on');
			set(hnd,'UserData',get(gcbo,'UserData'));
			mstat_cb('prepare_data');
			mstat_cb('prepare_axis');
			mstat_cb('redraw');

		%%%%%%%%%%%%%
		%Menu Edycja%
		%%%%%%%%%%%%%
		case 'savestate'
			name=get(Array_id,'Name');
			name=name(strfind(name,'-')+2:length(name));
			
			anames=get(SelectedArrayNameEdit_id,'UserData');
			aorder=get(AOrderAxes_id,'UserData');
			acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'String');
			gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'String');
			gorder=get(GOrderAxes_id,'UserData');
			gnames=get(SelectedGeneNameEdit_id,'UserData');
			glist=get(findobj(Array_id,'Tag','DatasetPanel'),'UserData');
			stat_struct=get(findobj(Array_id,'Tag','Select1D'),'UserData');

			restore=cell(1,9);
			restore{1}=name;
			restore{2}=data;
			restore{3}=anames;
			restore{4}=aorder;
			restore{5}=acnames;
			restore{6}=gcnames;
			restore{7}=gorder;
			restore{8}=gnames;
			restore{9}=glist;
			restore{10}=stat_struct;
			set(findobj(get(Array_id,'Child'),'Label','&Edycja'),'UserData',restore);		

		case 'restore'
			set(Array_id,'Pointer','watch');
			pause(0.01);

			restore=get(findobj(get(Array_id,'Child'),'Label','&Edycja'),'UserData');
			name=restore{1};
			data=restore{2};
			anames=restore{3};
			aorder=restore{4};
			acnames=restore{5};
			gcnames=restore{6};
			gorder=restore{7};
			gnames=restore{8};
			glist=restore{9};
			stat_struct=restore{10};

			set(Array_id,'UserData',data);
			set(SelectedGeneNameEdit_id,'UserData',gnames);
			set(SelectedArrayNameEdit_id,'UserData',anames);
			set(AOrderAxes_id,'UserData',aorder);
			set(GOrderAxes_id,'UserData',gorder);
			set(findobj(Array_id,'Tag','DatasetPanel'),'UserData',glist);
			set(findobj(Array_id,'Tag','ArrayClassPop'),'Value',1,'UserData',acnames);
			set(findobj(Array_id,'Tag','GeneClassPop'),'Value',1,'UserData',gcnames);
			set(findobj(Array_id,'Tag','Select1D'),'UserData',stat_struct);

			%%%%%%%%%Uaktywnienie menu i GUI%%%%%%%%%%%
			mstat_cb('update_missing');
			mstat_cb('reset_view');
			mstat_cb('do_menu');
			mstat_cb('do_gui');

			%%%%%%%%Przygotowanie i wywietlenie obrazu%%%%%%%%
			mstat_cb('prepare_data');
			mstat_cb('prepare_axis');			
			mstat_cb('redraw');

			%%%%%%%%%Zmiana belki tytuowej%%%%%%%%%
			set(Array_id,'Name',['MStat - ' name]);

			%%%%%%%%%Zmiana funkcji obsugi myszy%%%%%%%%%
			set(Array_id,'WindowButtonMotionFcn','mstat_mf');
			set(Array_id,'Pointer','arrow');

	end;
end;

