function [optimal_order,optimal_dispersion,centroids]=cluster_kmeans(data,clusters_count,centroid_type,dist_metric,klt,iterations_count,max_alg_iterations);

if nargin<2
	error('Missing input data');
elseif nargin==2
	centroid_type='kmeans';
	dist_metric='euclidean';
	klt=0;
	iterations_count=1;
	max_alg_iterations=50;
elseif nargin==3
	dist_metric='euclidean';
	klt=0;
	iterations_count=1;
	max_alg_iterations=50;
elseif nargin==4
	klt=0;
	iterations_count=1;
	max_alg_iterations=50;
elseif nargin==5
	iterations_count=1;
	max_alg_iterations=50;
elseif nargin==6
	max_alg_iterations=50;
end;

vectors_count=size(data,2);
if clusters_count<2 || clusters_count>vectors_count
	error('Wrong number of clusters');
end;

if klt~=0
	%org_data=data;
	data=data-repmat(mean(data,2),1,vectors_count);
	[u,s,v]=svd(data,0);
	data=s(1:klt,1:klt)*v(:,1:klt)';
end;

dimension=size(data,1);

centroids=zeros(dimension,clusters_count);
orders=ones(iterations_count,vectors_count);
dispersions=zeros(iterations_count,1);

rand('state',0);

for iteration=1:iterations_count
	alg_iteration=0;
	prev_dispersion=Inf;
	centroids=cluster_init(data,clusters_count);

	while 1
		alg_iteration=alg_iteration+1;

		order=cluster_centr2ord(data,centroids,dist_metric);
		[centroids,current_dispersion]=cluster_ord2centr(data,order,centroid_type,dist_metric,0);
%		rel_dispersion=(prev_dispersion-current_dispersion)/prev_dispersion;
%		if (rel_dispersion~=NaN & abs(rel_dispersion)<eps) | alg_iteration>=max_alg_iterations
		if (prev_dispersion~=Inf && prev_dispersion-current_dispersion==0) || alg_iteration>=max_alg_iterations
			break;
		else
			prev_dispersion=current_dispersion;
		end;
	end;
	dispersions(iteration)=current_dispersion;
	orders(iteration,:)=order;
end;

[optimal_dispersion,optimal_index]=min(dispersions);
%optimal_list=find(dispersions==optimal_dispersion);
optimal_order=cluster_reorder(orders(optimal_index,:))';

if nargout>1
%	if klt==0
		[centroids,optimal_dispersion]=cluster_ord2centr(data,optimal_order,centroid_type,dist_metric,1);
%	else
%		[centroids,optimal_dispersion]=cluster_ord2centr(org_data,optimal_order,centroid_type,dist_metric,1);
%	end;
end

return