#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "mex.h"

#define MAXLINE				65536
#define TYPE_MATRIX			0
#define TYPE_TABLE			1
#define	TYPE_ORDER_TABLE	2
#define	TYPE_R_TABLE		3

#define isnan(x) ((x) != (x))

int isStrNaN(char *str)
{
	int result=0;

	if (strlen(str)==4)
	{
		if (*str=='-' && (*(str+1)=='n' || *(str+1)=='N') && (*(str+2)=='a' || *(str+2)=='A') && (*(str+3)=='n' || *(str+3)=='N') )
			result=1;
	}
	else
	{
		if (strlen(str)==3)
		{
			if ((*str=='n' || *str=='N') && (*(str+1)=='a' || *(str+1)=='A') && (*(str+2)=='n' || *(str+2)=='N') )
				result=1;
		}
		else
		{
			if (strlen(str)==2)
			{
				if ((*str=='n' || *str=='N') && (*(str+1)=='a' || *(str+1)=='A'))
					result=1;
			}
		}
	}
	return(result);
}

int isStrInf(char *str)
{
	int result=0;

	if (strlen(str)==3 && (*str=='i' || *str=='I') && (*(str+1)=='n' || *(str+1)=='N') && (*(str+2)=='f' || *(str+2)=='F') )
		result=1;
	else if (strlen(str)==4 && *str=='-' && (*(str+1)=='i' || *(str+1)=='I') && (*(str+2)=='n' || *(str+2)=='N') && (*(str+3)=='f' || *(str+3)=='F') )
		result=1;

	return(result);
}

int isNumber(char *str)
{
	int result=1;

	while (*str)
	{
		if (isdigit(*str)==0 && *str!='-' && *str!='+' && *str!='.' && *str!='e' && *str!='E')
		{
			result=0;
			break;
		}
		str++;
	}

	return(result);
}

double getValue(char *num,double nanValue,double infValue)
{
	double value=nanValue;

	if (isStrInf(num)==1)
		value=(*num=='-')?-1*infValue:infValue;
	else if (isNumber(num)==1)
		value=atof(num);

	return(value);
}


void getSizeAndType(FILE *mtxFile,unsigned int *ySize,unsigned int *xSize,unsigned int *isTable)
{
	char	*line=NULL,*num=NULL;

	*ySize=0;
	*xSize=0;

	if (mtxFile!=NULL)
	{
		line=mxCalloc(MAXLINE+1,sizeof(char));

		if (fgets(line,MAXLINE,mtxFile)!=NULL)
			if ((num=strtok(line,"\t\n\r"))!=NULL)
			{
				*isTable=(isNumber(num)==0 && isStrNaN(num)!=1 && isStrInf(num)!=1)?TYPE_TABLE:TYPE_MATRIX;
				if (*isTable==TYPE_MATRIX)
				{
					(*ySize)++;
					(*xSize)++;
				}
				else
				{
					if (*line=='\t')
					{
						*isTable=TYPE_R_TABLE;
						(*xSize)++;
					}
				}
				while (strtok(NULL,"\t\n\r")!=NULL)
					(*xSize)++;
			}

		if (*isTable==TYPE_TABLE)
		{
			if (fgets(line,MAXLINE,mtxFile)!=NULL)
			{
				if (*line=='#')
					*isTable=TYPE_ORDER_TABLE;
				else
					(*ySize)++;
			}
		}

		while(fgets(line,MAXLINE,mtxFile)!=NULL)
			if ((num=strtok(line,"\t\n\r"))!=NULL)
				(*ySize)++;

		rewind(mtxFile);
		mxFree(line);
	}
}

mxArray* createStructure()
{
	mxArray *structure=NULL;
	char	**fnames=NULL;

	fnames=mxCalloc(6,sizeof(*fnames));
	fnames[0]="data";
	fnames[1]="column_names";
	fnames[2]="row_names";
	fnames[3]="name";
	fnames[4]="column_classes";
	fnames[5]="class_names";
	structure=mxCreateStructMatrix(1,1,6,(const char**)fnames);
	mxFree(fnames);

	return(structure);
}

mxArray *readMatrix(FILE *file,unsigned int ySize,unsigned int xSize)
{
	mxArray			*structure=NULL,*mtx=NULL;
	double			*mtxData=NULL;
	char			*line=NULL,*num=NULL;
	unsigned int	i=0,j=0;
	double			nanValue=mxGetNaN(),infValue=mxGetInf();

	structure=createStructure();
	mtx=mxCreateDoubleMatrix(ySize,xSize,mxREAL);
	mtxData=mxGetPr(mtx);

	line=mxCalloc(MAXLINE+1,sizeof(char));

	i=0;
	while (fgets(line,MAXLINE,file)!=NULL && i<ySize)
		if ((num=strtok(line,"\t\n\r"))!=NULL)
		{
			//*(mtxData+i)=(*num=='n' || *num=='N' || *(num+1)=='n' || *(num+1)=='N')? nanValue : atof(num);
			*(mtxData+i)=getValue(num,nanValue,infValue);
			j=1;
			while ((num=strtok(NULL,"\t\n\r"))!=NULL && j<xSize)
			{
				//*(mtxData+ySize*j+i)=(*num=='n' || *num=='N' || *(num+1)=='n' || *(num+1)=='N')? nanValue : atof(num);
				*(mtxData+ySize*j+i)=getValue(num,nanValue,infValue);
				j++;
			}
			i++;
	}
	mxFree(line);

	mxSetFieldByNumber(structure,0,0,mtx);
	mxSetFieldByNumber(structure,0,1,mxCreateCellMatrix(0,0));
	mxSetFieldByNumber(structure,0,2,mxCreateCellMatrix(0,0));
	mxSetFieldByNumber(structure,0,3,mxCreateString("NAME"));
	mxSetFieldByNumber(structure,0,4,mxCreateDoubleMatrix(0,0,mxREAL));
	mxSetFieldByNumber(structure,0,5,mxCreateCellMatrix(0,0));
	return(structure);
}

mxArray *getClassNames(char *str)
{
	mxArray			*classNames=NULL;
	unsigned int	classCounter=0,i=0;
	char			*ptr=NULL,*ptr2=NULL;

	ptr=str;
	while (*ptr!='\0')
	{
		if (*ptr=='#')
			classCounter++;
		ptr++;
	}

	if (classCounter>0)
	{
		classNames=mxCreateCellMatrix(classCounter,1);
		ptr=str+1;
		ptr2=ptr;
		for (i=0;i<classCounter;i++)
		{
			while (*ptr2!='#' && *ptr2!='\0')
				ptr2++;
			*ptr2='\0';
			mxSetCell(classNames,i,mxCreateString(ptr));
			ptr=ptr2+1;
			ptr2=ptr;
		}
	}
	else
		classNames=mxCreateCellMatrix(0,0);

	return(classNames);
}

mxArray *readTable(FILE *file,unsigned int ySize,unsigned int xSize,unsigned int tableType)
{
	mxArray			*structure=NULL,*mtx=NULL,*columnNames=NULL,*rowNames=NULL,*columnClasses=NULL,*classNames=NULL;
	double			*mtxData=NULL,*orderData=NULL;
	char			*line=NULL,*num=NULL,*name=NULL;
	unsigned int	i=0,j=0;
	double			nanValue=mxGetNaN(),infValue=mxGetInf();

	structure=createStructure();
	mtx=mxCreateDoubleMatrix(ySize,xSize,mxREAL);
	mtxData=mxGetPr(mtx);
	
	columnNames=mxCreateCellMatrix(xSize,1);
	rowNames=mxCreateCellMatrix(ySize,1);
	
	columnClasses=mxCreateDoubleMatrix(xSize,1,mxREAL);
	orderData=mxGetPr(columnClasses);
	for (i=0;i<xSize;i++)
		*(orderData+i)=1;

	line=mxCalloc(MAXLINE+1,sizeof(char));
	
	if (fgets(line,MAXLINE,file)!=NULL)
	{
		i=0;
		name=strtok(line,"\t\n\r");

		if (tableType==TYPE_R_TABLE)
		{
			mxSetFieldByNumber(structure,0,3,mxCreateString("NAME"));
			mxSetCell(columnNames,i++,mxCreateString(name));
		}
		else
		{
			if (*name=='#')
				mxSetFieldByNumber(structure,0,3,mxCreateString(name+1));
			else
				mxSetFieldByNumber(structure,0,3,mxCreateString("NAME"));
		}

		while ((name=strtok(NULL,"\t\n\r"))!=NULL && i<xSize)
			mxSetCell(columnNames,i++,mxCreateString(name));
	}

	if (tableType==TYPE_ORDER_TABLE)
	{
		if (fgets(line,MAXLINE,file)!=NULL)
		{
			classNames=getClassNames(strtok(line,"\t\n\r"));
			i=0;
			while ((num=strtok(NULL,"\t\n\r"))!=NULL && i<xSize)
				*(orderData+i++)=atof(num);
		}
	}
	else
			classNames=mxCreateCellMatrix(0,0);

	i=0;
	while (fgets(line,MAXLINE,file)!=NULL && i<ySize)
		if ((name=strtok(line,"\t\n\r"))!=NULL)
		{
			mxSetCell(rowNames,i,mxCreateString(name));
			j=0;
			while ((num=strtok(NULL,"\t\n\r"))!=NULL && j<xSize)
			{
				//*(mtxData+ySize*j+i)=(*num=='n' || *num=='N' || *(num+1)=='n' || *(num+1)=='N')? nanValue : atof(num);
				*(mtxData+ySize*j+i)=getValue(num,nanValue,infValue);
				j++;
			}
			i++;
	}
	mxFree(line);

	mxSetFieldByNumber(structure,0,0,mtx);
	mxSetFieldByNumber(structure,0,1,columnNames);
	mxSetFieldByNumber(structure,0,2,rowNames);
	mxSetFieldByNumber(structure,0,4,columnClasses);
	mxSetFieldByNumber(structure,0,5,classNames);
	return(structure);
}

mxArray* readData(char *fileName)
{
	mxArray			*structure=NULL;
	unsigned int	xSize=0,ySize=0,isTable=0;
	FILE			*file=NULL;
	
	if ((file=fopen(fileName,"rt"))!=NULL)
	{
		getSizeAndType(file,&ySize,&xSize,&isTable);
		if (ySize>0 && xSize>0)
		{
			if (isTable!=TYPE_MATRIX)
			{
			/*
				if (isTable==TYPE_TABLE)
					mexWarnMsgTxt("Loading table");
				else
					if (isTable==TYPE_R_TABLE)
						mexWarnMsgTxt("Loading R table");
					else
						mexWarnMsgTxt("Loading table with order");
			*/
				structure=readTable(file,ySize,xSize,isTable);
			}
			else
			{
				//mexWarnMsgTxt("Loading matrix");
				structure=readMatrix(file,ySize,xSize);
			}
		}
		fclose(file);
	}
	return(structure);
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
	mxArray			*mtx=NULL;
	char			*fileName=NULL;
	unsigned int	strLength=0;

	if (nrhs!=1)
		mexErrMsgTxt("One input argument required.");
	if (mxIsChar(prhs[0])!=1)
		mexErrMsgTxt("Input must be a string.");
	if (mxGetM(prhs[0]) != 1)
		mexErrMsgTxt("Input must be a row vector.");

	strLength=(mxGetM(prhs[0])*mxGetN(prhs[0]))+1;
	fileName=mxCalloc(strLength,sizeof(char));

	if (mxGetString(prhs[0],fileName,strLength)!=0) 
		mexWarnMsgTxt("Not enough space. String is truncated.");
	
	if ((mtx=readData(fileName))!=NULL)
		plhs[0]=mtx;
	else
		plhs[0]=createStructure();

	mxFree(fileName);
}
