#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mex.h"

#define MAXLINE 65536

void cellMtxSave(mxArray *cellMtx,char *fileName)
{
	FILE			*file;
	mxArray			*cell,*lineCell;
	char			*cellString;
	unsigned int	cellStringLength;
	unsigned int	xSize,ySize;
	int				i,j;

	if (cellMtx!=NULL && mxIsCell(cellMtx))
	{
		ySize=mxGetM(cellMtx);

		if (ySize!=0 && (file=fopen(fileName,"wt"))!=NULL)
		{
			cellString=mxCalloc(MAXLINE+1,sizeof(char));
			for (i=0;i<ySize;i++)
			{
				lineCell=mxGetCell(cellMtx,i);
				if (!mxIsCell(lineCell))
				{
					cellStringLength=mxGetM(lineCell)*mxGetN(lineCell)+1;
					cellStringLength=(cellStringLength>MAXLINE)? MAXLINE+1 : cellStringLength;
					mxGetString(lineCell,cellString,cellStringLength);
					fputs(cellString,file);
				}
				else
				{
					xSize=mxGetN(lineCell);
					for (j=0;j<xSize;j++)
					{
						cell=mxGetCell(lineCell,j);
						cellStringLength=mxGetM(cell)*mxGetN(cell)+1;
						cellStringLength=(cellStringLength>MAXLINE)? MAXLINE+1 : cellStringLength;
						mxGetString(cell,cellString,cellStringLength);
						fputs(cellString,file);
						if (j<xSize-1)
							fputs("\t",file);
					}				
				}
				fputs("\n",file);
			}
			fclose(file);
			if (cellString!=NULL)
				mxFree(cellString);
		}
	}
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
	mxArray			*mtx;
	char			*fileName;
	unsigned int	strLength;

	if (nrhs<2)
		mexErrMsgTxt("Two input arguments required.");
	if (mxIsChar(prhs[1])!=1 && mxGetM(prhs[1])!=1)
		mexErrMsgTxt("File name must be a string.");

	strLength=(mxGetM(prhs[1])*mxGetN(prhs[1]))+1;
	fileName=mxCalloc(strLength,sizeof(char));
	if (mxGetString(prhs[1],fileName,strLength)!=0) 
		mexWarnMsgTxt("File name string is truncated.");

	cellMtxSave(prhs[0],fileName);

	if (fileName!=NULL)
		mxFree(fileName);
}
