#include <stdio.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include "mex.h"

#define MAX_NEIGHBOURS	100
#define ETA				0.000000000001

void swap(double *array,int *index,int i, int j)
{
	double	array_tmp;
	int		index_tmp;
	
	array_tmp=array[i];
	array[i]=array[j];
	array[j]=array_tmp;

	if (index!=NULL)
	{
		index_tmp=index[i];
		index[i]=index[j];
		index[j]=index_tmp;
	}
}

void sort(double *array,int *index,int left,int right)
{
	int	i,last;
	
	if (left>=right)
		return;
		
	swap(array,index,left,(left+right)/2);
	last=left;
	for (i=left+1;i<=right;i++)
		if (array[i]<array[left])
				swap(array,index,++last,i);
	swap(array,index,left,last);
	
	sort(array,index,left,last-1);
	sort(array,index,last+1,right);
}

double getDoubleVal(double *data,int ySize, int y, int x)
{
	return(*(data+ySize*x+y));
}

void setDoubleVal(double val,double *data,int ySize, int y, int x)
{
	*(data+ySize*x+y)=val;
}

double* getMean(double *data,int ySize,int xSize)
{
	double			*mean=NULL;
	double			value=0,nanValue=mxGetNaN();
	int				i=0,j=0,n=0;

	mean=(double *)mxCalloc(ySize,sizeof(double));

	for (i=0;i<ySize;i++)
	{
		n=0;
		mean[i]=0;
		for (j=0;j<xSize;j++)
			if (!mxIsNaN(value=getDoubleVal(data,ySize,i,j)))
			{
				mean[i]+=value;
				n++;
			}
		mean[i]=(n!=0)?mean[i]/(double)n:nanValue;
	}
	return(mean);
}

double* getMedian(double *data,int ySize,int xSize)
{
	double			*median=NULL,*tmpData=NULL;
	double			value=0,nanValue=mxGetNaN();
	int				i=0,j=0,n=0,counter=0;

	median=(double *)mxCalloc(ySize,sizeof(double));

	for (i=0;i<ySize;i++)
	{
		n=0;
		median[i]=nanValue;
		for (j=0;j<xSize;j++)
			if (!mxIsNaN(value=getDoubleVal(data,ySize,i,j)))
				n++;
		if (n!=0)
		{
			tmpData=(double *)mxCalloc(n,sizeof(double));
			counter=0;
			for (j=0;j<xSize;j++)
				if (!mxIsNaN(value=getDoubleVal(data,ySize,i,j)))
					tmpData[counter++]=value;
			
			sort(tmpData,NULL,0,n-1);
//			mexPrintf("%d %d %d %d\n",n,n%2,n/2,n/2-1);
			median[i]=(n%2==0)?((tmpData[n/2]+tmpData[n/2-1])/2):(tmpData[n/2]);

			mxFree(tmpData);
		}
	}
	return(median);
}

int getRowIndex(double *data,int *rowIndex,int row1,int row2,int ySize,int xSize)
{
	int	i=0,n=0;

	for (i=0;i<xSize;i++)
		if (!mxIsNaN(getDoubleVal(data,ySize,row1,i)) && !mxIsNaN(getDoubleVal(data,ySize,row2,i)))
			rowIndex[n++]=i;

	if (n<xSize)
		rowIndex[n]=-1;

	return(n);
}

double getDistance(double *data,int *rowIndex,int row1,int row2,int ySize,int xSize,int minN)
{
	int			k=0,n=0;
	double		dist=0,mean1=0,mean2=0,sd1=0,sd2=0;

	if ((n=getRowIndex(data,rowIndex,row1,row2,ySize,xSize))<=minN)
		dist=DBL_MAX;
	else
	{
		mean1=0;
		mean2=0;
		for (k=0;(k<xSize && rowIndex[k]!=-1);k++)
		{
			mean1+=getDoubleVal(data,ySize,row1,rowIndex[k]);
			mean2+=getDoubleVal(data,ySize,row2,rowIndex[k]);
		}
		mean1/=n;
		mean2/=n;
				
		sd1=0;
		sd2=0;
		for (k=0;(k<xSize && rowIndex[k]!=-1);k++)
		{
			sd1+=(getDoubleVal(data,ySize,row1,rowIndex[k])-mean1)*(getDoubleVal(data,ySize,row1,rowIndex[k])-mean1);
			sd2+=(getDoubleVal(data,ySize,row2,rowIndex[k])-mean2)*(getDoubleVal(data,ySize,row2,rowIndex[k])-mean2);
		}
		sd1=sqrt(sd1/(n-1));
		sd2=sqrt(sd2/(n-1));

		for (k=0;(k<xSize && rowIndex[k]!=-1);k++)
			dist+=(getDoubleVal(data,ySize,row1,rowIndex[k])-mean1)*(getDoubleVal(data,ySize,row2,rowIndex[k])-mean2);

		dist=1-(dist/((n-1)*sd1*sd2));
		if (dist<=ETA)
			dist=DBL_MAX;
	}
	return(dist);
}

mxArray* knnRestore(double *data,int ySize,int xSize,int neighboursCount,int minN,double maxDist,int median,double *missingMask)
{
	mxArray		*restoredData=NULL;
	double		*restored=NULL;
	double		*dist=NULL,*mean=NULL;
	int			*columnIndex=NULL,*rowIndex=NULL;

	int			**neighbours=NULL;
	double		**neighboursDist=NULL;

	int			i=0,j=0,k=0,counter=0,counterN=0,rowRestored=0;
	double		value=0,distSum=0,neighbourDist=0,meanN=0;

	dist=(double*)mxCalloc(ySize,sizeof(double));

	rowIndex=(int*)mxCalloc(xSize,sizeof(int));
	columnIndex=(int*)mxCalloc(ySize,sizeof(int));
	
	neighbours=(int **)mxCalloc(ySize,sizeof(int *));
	neighboursDist=(double **)mxCalloc(ySize,sizeof(double *));
	for (i=0;i<ySize;i++)
	{
		neighbours[i]=(int*)mxCalloc(MAX_NEIGHBOURS,sizeof(int));
		neighboursDist[i]=(double *)mxCalloc(MAX_NEIGHBOURS,sizeof(double));
	}

	mean=(median==1)?getMedian(data,ySize,xSize):getMean(data,ySize,xSize);

	for (i=0;i<ySize;i++)
	{
		for (j=0;j<ySize;j++)
		{
			columnIndex[j]=j;
			dist[j]=(j!=i)?getDistance(data,rowIndex,i,j,ySize,xSize,minN):DBL_MAX;
		}

		sort(dist,columnIndex,0,ySize-1);

		for (j=0;(j<MAX_NEIGHBOURS && j<ySize);j++)
		{
			neighbours[i][j]=columnIndex[j];
			neighboursDist[i][j]=dist[j];
		}
	}

/*	restoredData=mxCreateDoubleMatrix(ySize,1,mxREAL);
	ptr=mxGetPr(restoredData);
	for (i=0;i<ySize;i++)
		setDoubleVal(dist[i],ptr,ySize,i,0);*/

	restoredData=mxCreateDoubleMatrix(ySize,xSize,mxREAL);
	restored=mxGetPr(restoredData);
	memcpy(restored,data,mxGetElementSize(restoredData)*mxGetNumberOfElements(restoredData));

	for (i=0;i<ySize;i++)
	{
		rowRestored=0;
		for (j=0;j<xSize;j++)
			if ((missingMask!=NULL && getDoubleVal(missingMask,ySize,i,j)==1) || (missingMask==NULL && mxIsNaN(getDoubleVal(data,ySize,i,j))))
			{
				counter=0;
				value=0;
				distSum=0;
				for (k=0;(k<MAX_NEIGHBOURS && k<ySize);k++)
					if (!mxIsNaN(getDoubleVal(data,ySize,neighbours[i][k],j)))
					{
						if ((counter>=2 && neighboursDist[i][k]>maxDist) || counter>=neighboursCount)
							break;
						neighbourDist=1/neighboursDist[i][k];
						distSum+=neighbourDist;
						value+=neighbourDist*getDoubleVal(data,ySize,neighbours[i][k],j)/mean[neighbours[i][k]];
						counter++;
					}
				value=(counter>0)?mean[i]*value/distSum:mean[i];
					
				setDoubleVal(value,restored,ySize,i,j);
				rowRestored=1;
			}
		if (rowRestored==1)
		{
			counterN++;
			meanN+=(double)counter;
		}
	}
	mexPrintf("\n%f\n",meanN/(double)counterN);

	mxFree(mean);
	mxFree(dist);

	mxFree(rowIndex);
	mxFree(columnIndex);

	for (i=0;i<ySize;i++)
	{
		mxFree(neighbours[i]);
		mxFree(neighboursDist[i]);
	}
	mxFree(neighbours);
	mxFree(neighboursDist);

	return(restoredData);
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
	double	*data=NULL,*ptr=NULL,*missingMask=NULL;
	int		xSize=0,ySize=0,minN=0,neighboursCount=4,median=0;
	double	maxDist=0.5,minNPercent=0.5;
	
	if (nlhs>1)
		mexErrMsgTxt("Too many output arguments");

	if (nrhs<1 || nrhs>6)
		mexErrMsgTxt("Wrong number of input arguments.");
	else
	{
		if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) || !(mxGetM(prhs[0])>0 && mxGetN(prhs[0])>0))
			mexErrMsgTxt("First argument must be a noncomplex double matrix.");		
		else
		{
			data=mxGetPr(prhs[0]);
			xSize=mxGetN(prhs[0]);
			ySize=mxGetM(prhs[0]);
		}
		if (nrhs>1)
		{ 
			if (!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1])==1 && mxGetN(prhs[1])==1))
				mexErrMsgTxt("neighboursCount must be a noncomplex scalar double.");
			else
				neighboursCount=(int)(*mxGetPr(prhs[1]));				
		}
		if (nrhs>2)
		{
			if (!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) || !(mxGetM(prhs[2])==1 && mxGetN(prhs[2])==1))
				mexErrMsgTxt("MinN must be a noncomplex scalar double.");
			else
			{
				minNPercent=*mxGetPr(prhs[2]);
				if (minNPercent>=1)
					minNPercent=0.5;
			}
				
		}
		if (nrhs>3)
		{
			if (!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) || !(mxGetM(prhs[3])==1 && mxGetN(prhs[3])==1))
				mexErrMsgTxt("MinCorr must be a noncomplex scalar double.");
			else
				maxDist=1-(*mxGetPr(prhs[3]));
		}
		if (nrhs>4)
		{
			if (!mxIsDouble(prhs[4]) || mxIsComplex(prhs[4]) || !(mxGetM(prhs[4])==1 && mxGetN(prhs[4])==1))
				mexErrMsgTxt("Median must be a noncomplex scalar double.");
			else
				median=(int)(*mxGetPr(prhs[4]));
		}
		if (nrhs>5)
		{
			if (!mxIsDouble(prhs[5]) || mxIsComplex(prhs[5]) || !(mxGetM(prhs[5])==ySize && mxGetN(prhs[5])==xSize))
				mexErrMsgTxt("MissingMask must be a noncomplex double matrix with the same size as the data matrix.");
			else
				missingMask=mxGetPr(prhs[5]);
		}
	}

	minN=xSize*minNPercent;
	if (minN<2)
		minN=2;
	plhs[0]=knnRestore(data,ySize,xSize,neighboursCount,minN,maxDist,median,missingMask);
}
