function plot_centroids_cb(action,fig);

if nargin==1
	PlotCentroids_id=gcbf;
else
	PlotCentroids_id=fig;
end;
Array_id=findobj(0,'Tag','ArrayView');
ViewMenu_id=findobj(PlotCentroids_id,'Tag','View');
ViewPlotMenu_id=findobj(ViewMenu_id,'Tag','PlotMenu');
ViewMeanMenu_id=findobj(ViewMenu_id,'Tag','MeanMenu');
ViewGroupMenu_id=findobj(ViewMenu_id,'Tag','GroupMenu');

switch action
	case 'plot_type'
		data_struct=get(PlotCentroids_id,'UserData');
		data_struct.plot=get(gcbo,'UserData');

		set(get(ViewPlotMenu_id,'Child'),'checked','off');
		set(gcbo,'checked','on');

		set(PlotCentroids_id,'UserData',data_struct);
		plot_centroids_cb('redraw');

	case 'norm'
		data_struct=get(PlotCentroids_id,'UserData');

		switch get(gcbo,'Checked')
			case 'on'
				data_struct.normalize=0;
				set(gcbo,'Checked','off');
			case 'off'
				data_struct.normalize=1;
				set(gcbo,'Checked','on');
		end;
		set(PlotCentroids_id,'UserData',data_struct);
		plot_centroids_cb('redraw');

	case 'centroid_type'
		data=get(Array_id,'UserData');
		gorder=get(findobj(Array_id,'Tag','GeneClassAxes'),'UserData');

		data_struct=get(PlotCentroids_id,'UserData');
		[data_struct.centroids,dispersion]=ord2centr(data',gorder,get(gcbo,'UserData'));
		data_struct.centroids=data_struct.centroids';

		set(get(ViewMeanMenu_id,'Child'),'checked','off');
		set(gcbo,'checked','on');

		set(PlotCentroids_id,'UserData',data_struct);
		plot_centroids_cb('redraw');

	case 'plot_thresholds'
		data_struct=get(PlotCentroids_id,'UserData');

		switch get(gcbo,'Checked')
			case 'on'
				data_struct.plot_thresholds=0;
				set(gcbo,'Checked','off');
			case 'off'
				data_struct.plot_thresholds=1;
				set(gcbo,'Checked','on');
		end;

		set(PlotCentroids_id,'UserData',data_struct);
		plot_centroids_cb('redraw');

	case 'set_thresholds'
		data_struct=get(PlotCentroids_id,'UserData');

		up=0;
		if strcmp(get(gcbo,'Tag'),'ThresholdUp')
			str=inputdlg('Warto:','MStat',1,cellstr(num2str(data_struct.threshold1)));
			num=str2num(char(str));
			if ~isempty(num)
				data_struct.threshold1=num;
			end;
		else
			str=inputdlg('Warto:','MStat',1,cellstr(num2str(data_struct.threshold2)));
			num=str2num(char(str));
			if ~isempty(num)
				data_struct.threshold2=num;
			end;
		end;

		set(PlotCentroids_id,'UserData',data_struct);
		plot_centroids_cb('redraw');

	case 'group'
		data_struct=get(PlotCentroids_id,'UserData');
		selected=get(gcbo,'UserData');

		if selected~=data_struct.selected

			if selected==0
				set(findobj(ViewPlotMenu_id,'Tag','BarPlot'),'Enable','off');
				set(findobj(ViewPlotMenu_id,'Tag','MeanBarPlot'),'Enable','off');
				data_struct.plot='line';
			else
				set(findobj(ViewPlotMenu_id,'Tag','BarPlot'),'Enable','on');
				set(findobj(ViewPlotMenu_id,'Tag','MeanBarPlot'),'Enable','on');
			end;

			if selected==-1
				data_struct.position=get(PlotCentroids_id,'Position');

				screen=get(0,'Screensize');

				groups_count=size(data_struct.centroids,1);
				columns=3;
				rows=ceil(groups_count/columns);

%				set(PlotCentroids_id,'Position',[0 0 screen(3) screen(4)-40]);
				if rows>1
					set(PlotCentroids_id,'Position',[0 0 min(400*columns+70,screen(3)) min(350*rows+70,screen(4)-40)]);
				else
					set(PlotCentroids_id,'Position',[0 0 min(400*columns+70,screen(3)) min(320,screen(4)-40)]);
				end;
			else
				if data_struct.selected==-1
					set(PlotCentroids_id,'Position',data_struct.position);
					delete(findobj(PlotCentroids_id,'Type','axes'));
					drawnow;
				end;
			end;

			data_struct.selected=selected;
			set(get(ViewGroupMenu_id,'Child'),'checked','off');
			set(gcbo,'checked','on');
			set(PlotCentroids_id,'UserData',data_struct);
			plot_centroids_cb('redraw');
		end;

	case 'redraw'
		data_struct=get(PlotCentroids_id,'UserData');

		centroids=data_struct.centroids;
		groups_count=size(centroids,1);
		vectors_count=size(centroids,2);

		colors_count=size(data_struct.color,1);

		if data_struct.normalize==1
			centroids=centroids./repmat(max(centroids,[],2),1,size(centroids,2));
		end;

		if data_struct.selected>=0
			delete(findobj(gca,'Type','line'));
			delete(findobj(gca,'Type','patch'));

			if data_struct.selected>0
				min_group=data_struct.selected;
				max_group=data_struct.selected;
			else
				min_group=1;
				max_group=groups_count;
			end;

			hold on;
			for i=min_group:max_group
				switch data_struct.plot
					case 'line'
						hnd=plot(centroids(i,:));

						cindex=mod(i,colors_count);
						if cindex==0
							cindex=colors_count;
						end;
						set(hnd,'Color',data_struct.color(cindex,:),'Marker','o','MarkerSize',2,'MarkerFaceColor',data_struct.color(cindex,:));

					case 'meanline'
						class_count=max(data_struct.aorder);
						mcentroid=zeros(1,class_count);
						for j=1:class_count
							index=find(data_struct.aorder==j);
							mcentroid(j)=mean(centroids(i,index));
						end;
						hnd=plot(mcentroid);
						
						cindex=mod(i,colors_count);
						if cindex==0
							cindex=colors_count;
						end;
						set(hnd,'Color',data_struct.color(cindex,:),'Marker','o','MarkerSize',2,'MarkerFaceColor',data_struct.color(cindex,:));

					case 'bar'
						for j=1:vectors_count
							fill([j-0.5;j-0.5;j+0.5;j+0.5],[0;centroids(i,j);centroids(i,j);0],data_struct.color(data_struct.aorder(j),:));
						end;
					case 'meanbar'
						class_count=max(data_struct.aorder);
						mcentroid=zeros(1,class_count);
						for j=1:class_count
							index=find(data_struct.aorder==j);
							mcentroid(j)=mean(centroids(i,index));
							fill([j-0.5;j-0.5;j+0.5;j+0.5],[0;mcentroid(j);mcentroid(j);0],data_struct.color(j,:));
						end;
				end;
			end;

			switch data_struct.plot
				case 'line'
					set(gca,'XLim',[1 vectors_count],'XTick',[1:vectors_count],'FontSize',8,'box','on');
					xlabel('Sample');
				case 'meanline'
					set(gca,'XLim',[0.5 class_count+0.5],'XTick',[1:class_count],'XTickLabel',data_struct.acnames,'box','on');
					xlabel('Group');
				case 'bar'
					set(gca,'XLim',[0.5 vectors_count+0.5],'XTick',[1:vectors_count],'box','on');
					xlabel('Sample');
				case 'meanbar'
					set(gca,'XLim',[0.5 class_count+0.5],'XTick',[1:class_count],'XTickLabel',data_struct.acnames,'box','on');
					xlabel('Group');
			end;
			if data_struct.plot_thresholds~=0 & data_struct.threshold1~=NaN
				hold on;
				hnd=plot(get(gca,'XLim'),[data_struct.threshold1 data_struct.threshold1],'-- ');
				set(hnd,'Color',[0.5 0.5 0.5]);	
				hold off;
			end;
			if data_struct.plot_thresholds~=0 & data_struct.threshold2~=NaN
				hold on;
				hnd=plot(get(gca,'XLim'),[data_struct.threshold2 data_struct.threshold2],'--');
				set(hnd,'Color',[0.5 0.5 0.5]);
				hold off;
			end;

			ylabel('Mean expression level');
			hold off;
		else
			columns=3;
			rows=ceil(groups_count/columns);
			max_val=max(centroids(:));
			min_val=min(centroids(:));

			for i=1:groups_count
				subplot(rows,columns,i);
				delete(findobj(gca,'Type','line'));
				delete(findobj(gca,'Type','patch'));

				switch data_struct.plot
					case 'line'
						hnd=plot(centroids(i,:));
						
						cindex=mod(i,colors_count);
						if cindex==0
							cindex=colors_count;
						end;
						set(hnd,'Color',data_struct.color(cindex,:),'Marker','o','MarkerSize',2,'MarkerFaceColor',data_struct.color(cindex,:));

					case 'meanline'
						class_count=max(data_struct.aorder);
						mcentroid=zeros(1,class_count);
						for j=1:class_count
							index=find(data_struct.aorder==j);
							mcentroid(j)=mean(centroids(i,index));
						end;
						hnd=plot(mcentroid);
						
						cindex=mod(i,colors_count);
						if cindex==0
							cindex=colors_count;
						end;
						set(hnd,'Color',data_struct.color(cindex,:),'Marker','o','MarkerSize',2,'MarkerFaceColor',data_struct.color(cindex,:));

					case 'bar'
						hold on;
						for j=1:vectors_count
							fill([j-0.5;j-0.5;j+0.5;j+0.5],[0;centroids(i,j);centroids(i,j);0],data_struct.color(data_struct.aorder(j),:));
						end;
						hold off;

					case 'meanbar'
						hold on;
						class_count=max(data_struct.aorder);
						mcentroid=zeros(1,class_count);
						for j=1:class_count
							index=find(data_struct.aorder==j);
							mcentroid(j)=mean(centroids(i,index));
							fill([j-0.5;j-0.5;j+0.5;j+0.5],[0;mcentroid(j);mcentroid(j);0],data_struct.color(j,:));
						end;
						hold off;
				end;

				range_min=min_val-0.1*abs(max_val-min_val);
				range_max=max_val+0.1*abs(max_val-min_val);
				switch data_struct.plot
					case 'line'
						set(gca,'XLim',[1 vectors_count],'YLim',[range_min range_max],'XTick',[1:vectors_count],'FontSize',8,'box','on');
					case 'meanline'
						set(gca,'XLim',[0.5 class_count+0.5],'YLim',[range_min range_max],'XTick',[1:class_count],'XTickLabel',data_struct.acnames,'box','on');
					case 'bar'
						set(gca,'XLim',[0.5 vectors_count+0.5],'YLim',[range_min range_max],'XTick',[1:vectors_count],'box','on');
					case 'meanbar'
						set(gca,'XLim',[0.5 class_count+0.5],'YLim',[range_min range_max],'XTick',[1:class_count],'XTickLabel',data_struct.acnames,'box','on');
				end;
				if data_struct.plot_thresholds~=0 & data_struct.threshold1~=NaN
					hold on;
					hnd=plot(get(gca,'XLim'),[data_struct.threshold1 data_struct.threshold1],'-- ');
					set(hnd,'Color',[0.5 0.5 0.5]);	
					hold off;
				end;
				if data_struct.plot_thresholds~=0 & data_struct.threshold2~=NaN
					hold on;
					hnd=plot(get(gca,'XLim'),[data_struct.threshold2 data_struct.threshold2],'--');
					set(hnd,'Color',[0.5 0.5 0.5]);
					hold off;
				end;
			end;
			if ~isempty(strfind(data_struct.plot,'mean'))
				suplabel('Group');
			else
				suplabel('Sample');
			end;
			suplabel('Mean expression level','y');
		end;
end;
return;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [centroids,dispersion]=ord2centr(data,order,centroid_type);
dimension=size(data,1);
clusters_count=max(order);
centroids=zeros(dimension,clusters_count);
dispersion=0;

for i=1:clusters_count
	cluster_vectors_list=find(order==i);
	
	if ~isempty(cluster_vectors_list)
		switch centroid_type
			case 'mean'
				centroids(:,i)=mean(data(:,cluster_vectors_list),2);
			case 'median'
				centroids(:,i)=median(data(:,cluster_vectors_list),2);
			case 'gmean'
				centroids(:,i)=gmean(data(:,cluster_vectors_list)')';
		end;
		dispersion=dispersion+sum(cluster_distance(centroids(:,i),data(:,cluster_vectors_list),'correlation',0));
	end;
end;
return;