function plot_cv_cb(action,fig);

if nargin==1
	PlotCV_id=gcbf;
else
	PlotCV_id=fig;
end;

PlotAxes_id=findobj(PlotCV_id,'Tag','PlotAxes');
ViewMenu_id=findobj(PlotCV_id,'Tag','ViewMenu');
DataMenu_id=findobj(PlotCV_id,'Tag','DataMenu');
InfoEdit_id=findobj(PlotCV_id,'Tag','InfoEdit');

switch action
	case 'save'
		data_struct=get(PlotCV_id,'UserData');
		figsave_cb(0,'noname_cv');

	case 'change_mode'
		selected=gcbo;
		set(get(ViewMenu_id,'Child'),'Check','off');
		set(selected,'Check','on');

		data_struct=get(PlotCV_id,'UserData');
		data_struct.mode=get(selected,'Tag');

		set(PlotCV_id,'UserData',data_struct);
		plot_cv_cb('redraw');

	case 'change_metric'
		selected=gcbo;
		set(get(DataMenu_id,'Child'),'Check','off');
		set(selected,'Check','on');

		data_struct=get(PlotCV_id,'UserData');
		data_struct.metric=get(selected,'Tag');

		set(PlotCV_id,'UserData',data_struct);
		plot_cv_cb('redraw');

	case 'redraw'
		data_struct=get(PlotCV_id,'UserData');
		switch data_struct.metric
			case 'cv'
				m=data_struct.m;
				cv=100*data_struct.sd ./m;
			case 'meddev'
				m=data_struct.med;
				cv=100*data_struct.meddev ./m;
			case 'iqrange'
				m=data_struct.med;
				cv=100*data_struct.iqrange ./m;
		end;
		name=data_struct.name;

		switch data_struct.mode
			case 'hist'
				hist(cv,30);
				xlabel('CV [%]');
				ylabel('Liczba wystpie');
				set(gca,'XLim',[0 max(cv)]);
				set(PlotCV_id,'Name',['Histogram wartoci CV: ' name]);

			case'sort'
				[cv index]=sort(cv);
				plot(cv);
				xlabel('Numer genu');
				ylabel('CV [%]');
				set(gca,'XLim',[0 length(cv)]);
				set(PlotCV_id,'Name',['Posortowane wartoci CV: ' name]);

			case'cvm'
				m=log10(m);
				[m,ind]=sort(m);
				cv=cv(ind);

%				hold on
%				plot(m,cv);
				s=lowess(m,cv,0.5,2);
				hnd=plot(m,s);
				set(hnd,'LineWidth',1.5);
%				hold off;
				ylabel('CV');
				xlabel('Poziom ekspresji');
				set(gca,'XLim',[min(m) max(m)],'YLim',[0 max(s)+5]);
				set(PlotCV_id,'Name',['CV vs rednia: ' name]);

			otherwise
				boxplot(cv,'notch','off','symbol','o','labels',' ');
	
				xlabel('');
				ylabel('CV [%]');
				set(PlotCV_id,'Name',['Boxplot wartoci CV: ' name]);
		end;
		set(PlotAxes_id,'Tag','PlotAxes');
		if isVersion(7.5)>0
			set(PlotAxes_id,'Position',[60 90 565 360]);
		end;

		str=sprintf('MEAN=%0.2f\t\tMEDIAN=%0.2f',nan_mean(cv),nan_median(cv));
		str=sprintf('%s\nMIN= %0.2f\t\tMAX=   %0.2f',str,min(cv),max(cv));
		str=sprintf('%s\nIQR= [%0.2f %0.2f]\tCONF=  [%0.2f %0.2f]',str,prctile(cv,25),prctile(cv,75),prctile(cv,2.5),prctile(cv,97.5));
		str=sprintf('%s\n#<30=%d\t\t#<40=%d\t\t#<50=%d',str,length(find(cv<30)),length(find(cv<40)),length(find(cv<50)));
		set(InfoEdit_id,'String',str);

end;