function [order,clusters]=plot_hierclust(z,param);

if nargin==1
	param=NaN;
end;
Array_id=findobj(0,'Tag','ArrayView');
Parent_tag=get(gcbf,'Tag');
screen_size=get(0,'ScreenSize');

vectors_count=size(z,1)+1;
zoom=4;
min_range=0;
color=ctable;
colors_count=size(color,1);


if strcmp(Parent_tag,'ACluster')
	margin_y=130;
	pos_x=30;
	pos_y=40;
	if zoom*vectors_count>screen_size(3)
		size_x=screen_size(3)-4;
		size_x=size_x-mod(size_x,zoom);
		range=size_x/zoom;
		scroller=1;
	else
		size_x=zoom*vectors_count;
		range=vectors_count;
		scroller=0;
	end;
	size_y=500;
	position=[pos_x screen_size(4)-(size_y+margin_y+pos_y) size_x+2 size_y+15+scroller*20+2+30];
	orientation='top';
	name='Klasteryzacja prbek';
	tag='PlotAHierClust';
else
	margin_y=130;
	pos_x=30;
	pos_y=40;
	if zoom*vectors_count>screen_size(4)-(margin_y+pos_y)
		size_y=screen_size(4)-(margin_y+pos_y);
		size_y=size_y-mod(size_y,zoom);
		range=size_y/zoom;
		scroller=1;
	else
		size_y=zoom*vectors_count;
		range=vectors_count;
		scroller=0;
	end;
	size_x=300;
	position=[pos_x screen_size(4)-(size_y+margin_y)+pos_y size_x+15+scroller*20+2 size_y+2+30];
	orientation='left';
	name='Klasteryzacja chech';
	tag='PlotGHierClust';
end;

HierClust_id=figure('Units','pixels', ...
		'NumberTitle','off', ...
		'Color',get(0,'defaultUicontrolBackgroundColor'),...
		'MenuBar','none', ...
		'Resize','off',...
		'Position',position,...
		'BackingStore','on',...
		'DoubleBuffer','on',...
		'Name',name,...
		'KeyPressFcn',@figure_kb_cb,...
		'Tag',tag);

if strcmp(Parent_tag,'ACluster')
	PlotAxes_id=axes('Parent',HierClust_id,...
			'Units','pixels',...
			'Tag','PlotAxes',...
			'Position',[1 16+20*scroller+30 size_x size_y]);

	OrderAxes_id=axes('Parent',HierClust_id,...
			'Units','pixels',...
			'Color',[0 0 0], ...
			'Tag','OrderAxes',...
			'Position',[1 2+20*scroller+30 size_x 15]);

	if scroller==1
		Scroll_id=uicontrol('Parent',HierClust_id, ...
			'Style','Slider', ...
			'Units','pixels', ...
			'Position',[1 2+30 size_x 19],...
			'BackgroundColor',[0.752941 0.752941 0.752941],...
			'Value',0,...
			'Min',0,...
			'Max',vectors_count-range,...
			'SliderStep',[1/(vectors_count-range) 1],...
			'Tag','Scroll',...
			'CallBack','plot_hierclust_cb(''scroll'')');	
	end;
else
	PlotAxes_id=axes('Parent',HierClust_id,...
			'Units','pixels',...
			'Tag','PlotAxes',...
			'Position',[1 2+30 size_x size_y]);

	OrderAxes_id=axes('Parent',HierClust_id,...
			'Units','pixels',...
			'Color',[0 0 0],...
			'YDir','Reverse',...
			'Tag','OrderAxes',...
			'Position',[size_x+1 2+30 15 size_y]);

	if scroller==1
		Scroll_id=uicontrol('Parent',HierClust_id, ...
			'Style','Slider', ...
			'Units','pixels', ...
			'Position',[size_x+1+16 2+30 20 size_y],...
			'BackgroundColor',[0.752941 0.752941 0.752941], ...
			'Value',vectors_count-range,...
			'Min',0,...
			'Max',vectors_count-range,...
			'SliderStep',[1/(vectors_count-range) 1],...
			'Tag','Scroll',...
			'CallBack','plot_hierclust_cb(''scroll'')');
	end;
end;

uicontrol('Parent',HierClust_id, ...
	'Units','pixels', ...
	'BackgroundColor',[1 1 1], ...
	'Position',[1 5 size_x 20], ...
	'HorizontalAlignment','left', ...
	'String','',...
	'Style','edit', ...
	'Tag','NameEdit');

%uicontrol('Parent',HierClust_id, ...
%	'Units','pixels', ...
%	'BackgroundColor',[1 1 1], ...
%	'Position',[1 5 size_x 20], ...
%	'HorizontalAlignment','left', ...
%	'String','',...
%	'Style','edit', ...
%	'Tag','NameEdit');

FileMenu_id=uimenu('Label','&Plik');
uimenu(FileMenu_id,'Label','&Zapisz...','CallBack','plot_hierclust_cb(''save'')');
uimenu(FileMenu_id,'Label','&Zamknij','Separator','on','CallBack','plot_hierclust_cb(''exit'')');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
axes(PlotAxes_id);
[h,t,order]=dendrogram(z,0,'ORIENTATION',orientation);

if ~isnan(param)
	if param>0
		clusters=cluster(z,'Cutoff',param,'Criterion','distance');
	else
		clusters=cluster(z,'MaxClust',-1*param);
	end;
else
	clusters=[];
end;

set(h,'Color',[0 0 0]);
set(PlotAxes_id,'Tag','PlotAxes');

axes(OrderAxes_id);
hold on;
if strcmp(Parent_tag,'ACluster')
	if ~isnan(param)
		aorder=clusters;
	else
		aorder=get(findobj(Array_id,'Tag','ArrayClassAxes'),'UserData');
	end;

	labels_order=aorder(order);
	anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');
	
	hnd=fill([0:vectors_count-1;0:vectors_count-1;1:vectors_count;1:vectors_count],repmat([0;1;1;0],1,vectors_count),'g');
	for i=1:vectors_count
		cindex=mod(labels_order(i),colors_count);
		if cindex==0
			cindex=colors_count;
		end;
		set(hnd(i),'FaceColor',color(cindex,:),'ButtonDownFcn','plot_hierclust_cb(''click'')','UserData',i);
	end;
else
	if ~isnan(param)
		gorder=clusters;
	else
		gorder=get(findobj(Array_id,'Tag','GeneClassAxes'),'UserData');
	end;
	labels_order=gorder(order);
	gnames=get(findobj(Array_id,'Tag','SelectedGeneNameEdit'),'UserData');

	hnd=fill(repmat([0;1;1;0],1,vectors_count),[0:vectors_count-1;0:vectors_count-1;1:vectors_count;1:vectors_count],'g');
	for i=1:vectors_count
		cindex=mod(labels_order(i),colors_count);
		if cindex==0
			cindex=colors_count;
		end;
		set(hnd(i),'FaceColor',color(cindex,:),'ButtonDownFcn','plot_hierclust_cb(''click'')','UserData',i);
	end;
end;
hold off;

data_struct.orientation=orientation;
data_struct.vectors_count=vectors_count;
data_struct.min_range=min_range;
data_struct.range=range;
data_struct.zoom=zoom;
data_struct.labels_order=labels_order;
data_struct.color=color;
data_struct.scroller=scroller;

set(HierClust_id,'UserData',data_struct);
plot_hierclust_cb('set_axes');

