function plot_pca_table(action,rows)

if nargin==0
	action='create';
	rows=[];
elseif nargin==1
	rows=[];	
end;

import com.mathworks.mwt.*;
import java.awt.* ;
import javax.swing.*;
import javax.swing.table.*;

table_id='PCATable';
table_name='Lista PCA';

Array_id=findobj(0,'Tag','ArrayView');
if ~isempty(Array_id)
	%frame=findobj('Tag',table_id);
	       
	switch action
		case 'create'
            PlotPCA_id=gcbf;
            table_id=[table_id num2str(PlotPCA_id)];
            
            frame=findobj(0,'Tag',table_id);
            if ~isempty(frame)
                figure(frame);
            else
                set(Array_id,'Pointer','watch');
                pause(0.01);

                config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');

                %%%%%%%%%%%%%%%%
                %Tworzenie okna%
                %%%%%%%%%%%%%%%%
                frame=figure('Units','pixels', ...
                            'Color',config.bg_color, ...
                            'Visible','on',...
                            'MenuBar','none', ...
                            'NumberTitle','off', ...
                            'Resize','off',...
                            'Name',table_name, ...
                            'KeyPressFcn',@figure_kb_cb,...
                            'Tag',table_id);
        
                table=create_table(frame);
                fill_table(table);
                set(frame,'UserData',table);

                control_panel=create_control_panel(frame);

                main_panel=JPanel;
                main_panel.setLayout(BorderLayout);

                table_panel=JScrollPane;
                scrollayout=ScrollPaneLayout;
                table_panel.setLayout(scrollayout);
                viewport=get(scrollayout,'ViewPort');
                viewport.setView(table);
                table.requestFocus

                main_panel.add(table_panel,BorderLayout.CENTER);
                main_panel.add(control_panel,BorderLayout.SOUTH);

                [size_x,size_y]=get_table_size(table);
                screen_size=get(0,'Screensize');
                size_y=min(size_y,screen_size(4)-75);
                set(frame,'Position',[5 (screen_size(4)-size_y-30) size_x size_y])

                javacomponent(main_panel,[0 0 size_x size_y],frame);

                hnd_struct.parent=PlotPCA_id;
                hnd_struct.table=table;
                set(frame,'UserData',hnd_struct);
                
                %set_selection(table);
                set(Array_id,'Pointer','arrow');
            end

		case 'reorder'
            frame=gcbf;
			fill_table(get(frame,'UserData'));

        case 'resize'
            frame=gcbf;            
			resize_frame(frame);
            
        case 'remove'
            frame=gcbf;            
            if ~isempty(rows)		
				remove_rows(frame,rows);
            end;
		
		case 'set_selection'
			set_selection(get(frame,'UserData'));			
	end;
end;
return;


%%%%%%%%%%%%%%%%%%
%Tworzenie tabeli%
%%%%%%%%%%%%%%%%%%
function table=create_table(frame)

import javax.swing.*;
import javax.swing.table.*;

Array_id=findobj(0,'Tag','ArrayView');
anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');

table_model=DefaultTableModel({'#','Prbka','Klasa','Nazwa klasy'},length(anames));
table=JTable(table_model);
%table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

h=handle(table,'callbackproperties');
set(h,'MousePressedCallback',{@mouse_cb,table,frame});

columnModel=table.getColumnModel;
columns_count=table.getColumnCount;

column=columnModel.getColumn(0);
column.setPreferredWidth(40);
column.setCellRenderer(table.getTableHeader.getDefaultRenderer);
textfield=JTextField;
textfield.setEditable(0);
table.getColumnModel.getColumn(0).setCellEditor(DefaultCellEditor(textfield));


column=columnModel.getColumn(1);
column.setPreferredWidth(250);

column=columnModel.getColumn(2);
column.setPreferredWidth(80);
renderer=DefaultTableCellRenderer;
renderer.setHorizontalAlignment(SwingConstants.CENTER);
column.setCellRenderer(renderer);

column=columnModel.getColumn(3);
column.setPreferredWidth(140);

return;

%%%%%%%%%%%%%%%%%%%%
%Wypelnienie tabeli%
%%%%%%%%%%%%%%%%%%%%
function fill_table(table)

Array_id=findobj(0,'Tag','ArrayView');

anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');
aorder=get(findobj(Array_id,'Tag','ArrayClassAxes'),'UserData');
acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'UserData');
		
table_model=table.getModel;
rows_count=table.getRowCount;

if length(acnames)==1 && strcmp(acnames{1},'Brak podziau na klasy')==1
	no_groups=1;
else
	no_groups=0;
end;

for i=1:rows_count
	table_model.setValueAt(sprintf('%d',i),i-1,0);
	table_model.setValueAt(anames{i},i-1,1);
	table_model.setValueAt(sprintf('%d',aorder(i)),i-1,2);
	if no_groups==1
		table_model.setValueAt('      -',i-1,3);
	else
		table_model.setValueAt(acnames{aorder(i)},i-1,3);
	end;
end;
return;			

%%%%%%%%%%%%%%%
%Set selection%
%%%%%%%%%%%%%%%
function set_selection(table)

Array_id=findobj(0,'Tag','ArrayView');
config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
			
if config.view_struct.selected_array>0
	table.changeSelection(config.view_struct.selected_array-1,0,false,false);
	table.repaint;
	drawnow;
end;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Tworzenie panelu kontrolnego%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function control_panel=create_control_panel(frame)

import javax.swing.*;
import java.awt.* ;

table=get(frame,'UserData');

control_panel=JPanel;
box_layout=BoxLayout(control_panel,BoxLayout.X_AXIS);
control_panel.setLayout(box_layout);

control_panel.add(Box.createRigidArea(Dimension(2,30)));
			
edit=handle(JTextField(),'CallbackProperties');
edit.setMinimumSize(Dimension(200,20));
edit.setPreferredSize(Dimension(200,20));
edit.setMaximumSize(Dimension(200,20));
set(edit,'ActionPerformedCallback',{@buttons_cb,edit,table,frame,'Szukaj'});
control_panel.add(edit);
				
control_panel.add(Box.createHorizontalGlue);
return;
			
%%%%%%%%%%%%%%%%%
%Skalowanie okna%
%%%%%%%%%%%%%%%%%
function [size_x,size_y]=get_table_size(table)

columns_count=table.getColumnCount;
rows_count=table.getRowCount;

table_width=510;
table_height=(1+rows_count)*table.getRowHeight;

size_x=table_width;
size_y=table_height+43;

return;

function resize_frame(frame)

table=get(frame,'UserData');

[size_x,size_y]=get_table_size(table);
screen_size=get(0,'Screensize');
size_y=min(size_y,screen_size(4)-75);
set(frame,'Position',[5 (screen_size(4)-size_y-30) size_x size_y]);
set(get(frame,'Child'),'Position',[0 0 size_x,size_y]);

return;

%%%%%%%%%%%%%
%Remove rows%
%%%%%%%%%%%%%
function remove_rows(frame,rows)

table=get(frame,'UserData');
			
rows=sort(rows,'descend');
table_model=table.getModel;
for i=1:length(rows)
	table_model.removeRow(rows(i)-1);
end;
table.updateUI;
drawnow;
                
resize_frame(frame);
return;
			
%%%%%%%%%%%%%%%%
%Mouse callback%
%%%%%%%%%%%%%%%%
function mouse_cb(varargin)

src=varargin{1};
evt=varargin{2};
table=varargin{3};
frame=varargin{4};

if evt.getButton==1 && evt.getClickCount==1

    hnd_struct=get(frame,'UserData');
    PlotPCA_id=hnd_struct.parent;
    if ~isempty(PlotPCA_id)
        data_struct=get(PlotPCA_id,'UserData');
            
        data_struct.selected=table.getSelectedRows+1;
		set(PlotPCA_id,'UserData',data_struct);
		plot_pca_cb('redraw',PlotPCA_id);
    end;

    Array_id=findobj(0,'Tag','ArrayView');
    if ~isempty(Array_id)
		config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
    
		index=table.getSelectedRow;
		if index>=0
			array_nr=index+1;
			config.view_struct.selected_array=array_nr;
			config.view_struct.selected_gene=max(config.view_struct.selected_gene,1);
			set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
			mstat_cb('set_selected_array');
		end;
        figure(PlotPCA_id);
		figure(frame);
    end;
end;	

%%%%%%%%%%%%%%
%Key callback%
%%%%%%%%%%%%%
function kb_cb(src,evnt)

%Array_id=gcbf;
%data=get(Array_id,'UserData');
%config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');

%evnt

return;
        
%%%%%%%%%%
%Callback%
%%%%%%%%%%
function buttons_cb(varargin)

button=varargin{3};
table=varargin{4};
frame=varargin{5};
cmd=varargin{6};

Array_id=findobj(get(0,'Child'),'Tag','ArrayView');
if ~isempty(Array_id)
	switch char(cmd)
		case 'Szukaj'
			table_cb(table,'Szukaj',button);
	end;
end;
return;

