/* wrappers.c: wrapper functions for standard library functions that
 * check for successful outcome.
 */
/* Rune Lyngs, August 2003 */
#include <stdio.h>
#include <stdlib.h>
#include "wrappers.h"
#include "mex.h"
#ifdef MPROFILING
#include "splay.h"

static splaytree *allocated = NULL;
int _inside_wrapper = 0;

/* xmalloc(n): Tries to allocate n bytes of memory; if unsuccessful an
 * error is reported.
 */
typedef struct _mprofile_info{
  void *addr;
  char *id;
} mprofile_info;

static void *xmalloc(size_t n)
{
  void *adr = malloc(n);

  if (adr == NULL){
    fprintf(stderr, "Alzheimer's error\n");
    exit(1);
  }

  return adr;
}

static int cmp_addr(mprofile_info *a, mprofile_info *b)
{
  return a->addr < b->addr;
}
#endif

/* xmalloc(n): Tries to allocate n bytes of memory; if unsuccessful an
 * error is reported. 
 */
#ifdef MPROFILING
void *wmalloc(size_t n, char *id)
#else
void *wmalloc(size_t n)
#endif
{
//  void *adr = malloc(n);
  void *adr = (void *)mxMalloc(n);
#ifdef MPROFILING
  mprofile_info *mi;
#endif

  if (adr == NULL){
    fprintf(stderr, "Alzheimer's error\n");
    exit(1);
  }

#ifdef MPROFILING
  if ((id != NULL) && (!_inside_wrapper)){
    _inside_wrapper = 1;
    if (allocated == NULL)
      allocated = splay_create((int (*)(void *, void *))cmp_addr, NULL);
    mi = (mprofile_info *)xmalloc(sizeof(mprofile_info));
    mi->addr = adr;
    mi->id = id;
    splay_insert(allocated, mi);
    _inside_wrapper = 0;
  }
#endif

  return adr;
}

void wfree(void *adr)
{
#ifdef MPROFILING
  mprofile_info mi;

  if (!_inside_wrapper){
    _inside_wrapper = 1;
    mi.addr = adr;
    if ((allocated == NULL) || (!splay_member(allocated, &mi))){
      fprintf(stderr, "Attempt to free non-allocated memory.\n");
      exit(1);
    }
    free(splay_delete(allocated, &mi));
    _inside_wrapper = 0;
  }
#endif
//  free(adr);
  mxFree(adr);
}

#ifdef MPROFILING
static void print_mi(mprofile_info *mi)
{
  if (mi->id != NULL)
    printf(mi->id);
}

void print_allocated(void)
{
  if ((allocated == NULL) || splay_empty(allocated))
    printf("No memory remains allocated\n");
  else
    splay_listtreeelements(allocated, (void (*)(void *))print_mi, ", ");
}
#endif
