function [w,p]=mann_whitney(data,order,class_index)

if nargin<3
	class_index=[min(order) max(order)];
end;

dimension=size(data,1);

ind_x=find(order==class_index(1));
ind_y=find(order==class_index(2));

nx=length(ind_x);
ny=length(ind_y);

data=[data(:,ind_x) data(:,ind_y)];
labels=[repmat(1,dimension,nx) repmat(2,dimension,ny)];
ranks=[repmat(1:(nx+ny),dimension,1)];

[data,index]=sort(data,2);
for i=1:dimension
	labels(i,:)=labels(i,index(i,:));
end;

[tied_y,tied_x]=find(~diff(data,[],2));
[tied_y,index]=sort(tied_y);
tied_x=tied_x(index);
tied_ind=unique(tied_y);
notied_ind=1:dimension;
notied_ind(tied_ind)=[];

tieadj=zeros(dimension,1);
while ~isempty(tied_y)
	current_tied_y=tied_y(1);
	current_tied_x=tied_x(1);
	tied_count=1+sum(data(current_tied_y,current_tied_x+1:end)==data(current_tied_y,current_tied_x));
	ranks(current_tied_y,current_tied_x:current_tied_x+tied_count-1)=ranks(current_tied_y,current_tied_x)+(tied_count-1)/2;
	tieadj(current_tied_y)=tieadj(current_tied_y)+tied_count*(tied_count-1)*(tied_count+1)/2;
	tied_y(1:tied_count-1)=[];
	tied_x(1:tied_count-1)=[];
end;

if nx<=ny
	ns=nx;
	label=1;
else
	ns=ny;
	label=2;
end;

w=zeros(dimension,1);
for i=1:dimension
	w(i)=sum(ranks(i,find(labels(i,:)==label)));
end;


if nargout>1
	wmean=ns*(nx+ny+1)/2;

	p=zeros(dimension,1);

	%ranks(notied_ind(1))
	if ns < 10 & (nx+ny) < 20     % Use the sampling distribution of W.
		if ~isempty(notied_ind)
			allpos=nchoosek(ranks(notied_ind(1),:),ns);
			sumranks=sum(allpos,2);
			np=length(sumranks);
			for i=1:length(notied_ind)
				if w(notied_ind(i))<wmean
					p(notied_ind(i))=2*length(find(sumranks<=w(notied_ind(i))))./np;
				else
					p(notied_ind(i))=2*length(find(sumranks>=w(notied_ind(i))))./np;
				end
				p(notied_ind(i))=min(p(notied_ind(i)),1);        % p>1 means w is in the middle and double-counted
			end;
		end;
		if ~isempty(tied_ind)
			for i=1:length(tied_ind)
				allpos=nchoosek(ranks(tied_ind(i),:),ns);
				sumranks=sum(allpos,2);
				np=length(sumranks);
				if w(tied_ind(i))<wmean
					p(tied_ind(i))=2*length(find(sumranks<=w(tied_ind(i))))./np;
				else
					p(tied_ind(i))=2*length(find(sumranks>=w(tied_ind(i))))./np;
				end
				p(tied_ind(i))=min(p(tied_ind(i)),1);        % p>1 means w is in the middle and double-counted
			end;
		end;
	else
		tiescor=2*tieadj/((nx+ny)*(nx+ny-1));
		wvar=nx*ny*((nx + ny + 1)-tiescor)/12;
		wc=w-wmean;
		z=(wc-0.5*sign(wc))./sqrt(wvar);
		p=normcdf(z,0,1);
		p=2*min(p,1-p);
	end;
end;
w=abs(w-(ns*(nx+ny+1)/2));