function [p,adj_p,index]=selection1D(data,order,selection_method,threshold_method,threshold,iterations,pn)

%t = cputime;

dimension=size(data,1);

if nargin<3
	selection_method='anova';
	threshold_method='none';
	threshold=1;
	iterations=0;
	pn=0;
elseif nargin==3
	threshold_method='none';
	threshold=1;
	iterations=0;
	pn=0;
elseif nargin==4
	threshold=1;
	iterations=0;
	pn=0;
elseif nargin==5
	iterations=0;
	pn=0;
elseif nargin==6
	pn=0;
end;

selection_method=lower(selection_method);
threshold_method=lower(threshold_method);

if iterations==0
	disp(sprintf('Stat: %s',selection_method));
	switch selection_method
		case 'anova'
			[F,ts,p]=anova_test(data,order);
		case 'anova2'
			[F,p]=mwanova(data,order);
		case 'kw'
			p=kw(data,order);
		case 'satterthwaite'
			[t,p]=satterthwaite(data,order);
		case 'mann-whitney'
			[t,p]=mann_whitney(data,order);
		case 'pttest'
			[t,p]=t_testp(data);
		case 'pwilcoxon'
			p=wilcoxonp(data);
	end;
else
	p=resampling_test(data,order,iterations,selection_method,pn);
end;

p(isnan(p))=1;
p(isinf(p))=1;

[p,index]=sort(p);
f=fdr_bh(p,1);
adj_p=f;

switch threshold_method
	case 'none'
		adj_p=f;
		ind=1:length(adj_p);

	case 'max_pval'
		adj_p=f;
		ind=find(p<=threshold);

	case 'bonferroni'
%		threshold=threshold/dimension;
		adj_p=min(p*dimension,1);
		ind=find(adj_p<=threshold);

	case 'sidak'
%		threshold=1-(1-threshold)^(1/dimension)
		adj_p=min(1-(1-p).^dimension,1);
		ind=find(adj_p<=threshold);

	case 'holm'
		adj_p=zeros(size(p));
		adj_p(1)=p(1)/dimension;
		for i=2:dimension
			adj_p(i)=max(min((dimension-i+1)*p(i),1),adj_p(i-1));
		end;
		ind=find(adj_p<=threshold);

	case 'fdr_bh'
		adj_p=f;
		%adf_p=mafdr(p,'BHFDR',true);
		ind=find(adj_p<=threshold);

	case 'fdr_bh_tst'
		adj_p=f;
		r=length(find(adj_p<=(threshold/(1+threshold))));
		switch r
			case 0
				ind=[];
			case dimension
				ind=1:dimension;
			otherwise
				m_m0=dimension/((dimension-r)*(1+threshold));
				if m_m0>1
					adj_p=adj_p/m_m0;
				end;
				ind=find(adj_p<=threshold);
		end;

	case 'fdr_storey'
%		alfa=0.5;
%		r=length(find(p<=alfa));
%		m_m0=dimension/((dimension-r)/(1-alfa))
%		adj_p=f/m_m0;

		adf_p=mafdr(p);
		ind=find(adj_p<=threshold);
		
	otherwise
		return;
end;

p=p(ind);
index=index(ind);
adj_p=adj_p(ind);

%e = cputime-t