function [filenames, pathname,fileformat] = uigetfilej(filters,name,path,multiple)
% This is a Java interfaced version of UIGETFILES, that brings multiple file
% open dialog box. 
%
% [filenames, pathname] = uigetfiles; displays a dialog box file browser
% from which the user can select multiple files.  The selected files are
% returned to FILENAMES as an arrayed strings. The directory containing
% these files is returned to PATHNAME as a string. 
%
% A successful return occurs only if the files exist.  If the user selects
% a  file that does not exist, an error message is displayed to the command
% line.  If the Cancel button is selected, zero is assigned to FILENAMES
% and current directory is assigned to PATHNAME. 
% 
% This program has an equivalent function to that of a C version of
% "uigetfiles.dll" downloadable from www.mathworks.com under support, file
% exchange (ID: 331). 
%
% It should work for matlab with Java 1.3.1 or newer.
%
% Shanrong Zhang
% Department of Radiology
% University of Texas Southwestern Medical Center
% 02/09/2004
%
% email: shanrong.zhang@utsouthwestern.edu

% mainFrame = com.mathworks.ide.desktop.MLDesktop.getMLDesktop.getMainFrame;


if nargin==0
	filters=[];
	name=[];
	path='.';
	multiple=0;
elseif nargin==1
	name=[];
	path='.';
	multiple=0;
elseif nargin==2
	path='.';
	multiple=0;
else
	multiple=0;
end;


javaaddpath( fullfile(eval('pwd'),'java') );

import mscanlib.gui.*;

try
	filechooser = mscanlib.gui.MScanJFileChooser(path);
catch
	filechooser=[];

	s=lasterror();
	emsg = {''; ['Java Exception has been caught:']; s.message; ''};
	for i=1:size(s.stack,1)
		emsg = [emsg; [s.stack(i).file '::' s.stack(i).name '() at ' num2str(s.stack(i).line)]];
	end
	if size(s.stack,1) > 0
		emsg = [emsg; ''];
	end
	errordlg(emsg,'Error','modal');
end;

if ~isempty(filechooser)
	methods(filechooser)
	if multiple~=0
		filechooser.setMultiSelectionEnabled(true);
	end;
	filechooser.setFileSelectionMode(filechooser.FILES_ONLY);

	if ~isempty(filters)
		for i=size(filters,1):-1:1
			ext=char(filters(i,1));
			if ext(1:2)=='*.'
				ext=ext(3:end);
			end;
			if size(filters,2)==1
				description=[ '(' char(filters(i,1)) ')' ];
			else
				description=char(filters(i,2));
			end;
			filechooser.addFileFilter({ext description});
		end;
	end

	selectionStatus=filechooser.showOpenDialog(com.mathworks.mwswing.MJFrame); 

	if selectionStatus == filechooser.APPROVE_OPTION
		pathname=[char(filechooser.getCurrentDirectory.getPath),java.io.File.separatorChar]
		fileformat=filechooser.getFileFormat+1;

		if multiple==1
			selectedfiles=filechooser.getSelectedFiles;
		else
			selectedfiles=filechooser.getSelectedFile;
		end;

		for k = 1:1:size(selectedfiles)
			filenames(k)=selectedfiles(k).getName;
		end

		filenames=char(filenames);  
	else
		pathname=pwd;
		filenames=0;
		fileformat=-1;
	end
end;
