/* reliability.h: defintions and prototypes for functions creating an
 * interface to functions from the GNU multiprecission library, thus
 * making it easy to replace e.g. rationals with floats, or even with
 * non-GMP types like doubles.
 */
/* Rune Lyngs, February 2003 */

#ifndef _GMP_INTEERFACE_H
#define _GMP_INTEERFACE_H

#ifdef USE_GMP
#include "gmp.h"
typedef mpq_t mp_type;
#else
typedef double mp_type[1];
#endif

/* Naming conventions as for mpf functions - see info for gmp for usage */
void gmpi_init(mp_type X);
void gmpi_clear(mp_type X);
void gmpi_set(mp_type ROP, mp_type OP);
void gmpi_set_ui(mp_type ROP, unsigned long int OP);
void gmpi_set_si(mp_type ROP, signed long int OP1, unsigned long int OP2);
void gmpi_set_d(mp_type ROP, double OP);
void gmpi_init_set_ui(mp_type ROP, unsigned long int OP);
void gmpi_add(mp_type SUM, mp_type OP1, mp_type OP2);
void gmpi_inc(mp_type OP);
void gmpi_sub(mp_type SUM, mp_type OP1, mp_type OP2);
void gmpi_inv(mp_type OP);
void gmpi_mul(mp_type ROP, mp_type OP1, mp_type OP2);
void gmpi_div(mp_type ROP, mp_type OP1, mp_type OP2);
double gmpi_get_d(mp_type OP);
void gmpi_print(mp_type OP);

#endif
