/* lsd_polynomial.h: Function prototype and global definitions for
 * various functions for doing one dimensional interpolated loess
 * fitting.
 */
/* Rune Lyngs, June 2003 */
#include "loess.h"

#ifndef _IPLOESS_H
#define _IPLOESS_H

/* Robust loess fitting iteration stop when the maximum change in any
 * weight between two consecutive iterations becomes less than
 * _IPLOESS_ROBUSTSTOP or when _IPLOESS_ROBUSTMAXITERATIONS iterations
 * is exceeded.
 */
#define _IPLOESS_ROBUSTSTOP 0.0001
#define _IPLOESS_ROBUSTMAXITERATIONS 100

typedef struct {
  int n;        /* Number of interpolation intervals */
  double **p;   /* Polynomial coefficients for each interval */
  double left;  /* Left border of interpolated interval */
  double right; /* Right border of interpolated interval */
  int lambda;   /* Degree of polynomials */
} IPLoess;

/* Descriptions of functions precede the implementation of the
 * function in iploess.c.
 */
IPLoess *iploess(Loess *l, int n, double a, double b);
IPLoess *iploess_robust(Loess *l, int n, double a, double b);
double iploess_evaluate(IPLoess *ipl, double x);
void iploess_free(IPLoess *ipl);
double iploess_isd(IPLoess *ipl1, IPLoess *ipl2, double a, double b);

#endif
