/* splay.h: header file for splay.c; definition of specific splay tree
 * data types.
 */
#ifndef _SPLAY_H
#define _SPLAY_H

typedef struct _splaytree {
  struct _splaynode *root;
  int (*less_than)(void *, void *);
  void (*update)(void *, void *, void *);
} splaytree;

typedef struct _splaynode {
  struct _splaynode *left;
  struct _splaynode *right;
  struct _splaynode *parent;
  void *item;
} splaynode;

/* Function declarations */
void splay_preallocatenodes(int n);
void splay_freeunusednodes();
splaytree *splay_create(int (*less_than)(void *, void *),
			void (*update)(void *, void *, void *));
void splay_deletetree(splaytree *tree, void (*delete_item)(void *));
void splay_destroytree(splaytree *tree, void (*destroy_item)(void *));
void splay_insert(splaytree *tree, void *item);
void *splay_delete(splaytree *tree, void *item);
void splay_deleterange(splaytree *tree,
		       void *item1, int (*less_than1)(void *, void *),
		       void *item2, int (*less_than2)(void *, void *),
		       void (*delete_item)(void *));
int splay_member(splaytree *tree, void *item);
void *splay_access(splaytree *tree, void *item);
void *splay_accesssmaller(splaytree *tree, void *item);
void *splay_accesslarger(splaytree *tree, void *item);
void *splay_evaluate(splaytree *tree, void *item,
		     void *(*combine)(void *, void *, void *));
splaytree *splay_join(splaytree *tree1, splaytree *tree2);
splaytree *splay_split(splaytree *tree, void *item);
void *splay_min(splaytree *tree);
void *splay_max(splaytree *tree);
void *splay_deletemin(splaytree *tree);
void *splay_deletemax(splaytree *tree);
int splay_empty(splaytree *tree);
void splay_printtree(splaytree *tree, void (*print_item)(void *));
void splay_listtreeelements(splaytree *tree, void (*print_item)(void *),
			    char *separator);
void check_consistency(splaytree *t);

#endif
